(function (factory) {
    if (typeof module === "object" && typeof module.exports === "object") {
        var v = factory(require, exports);
        if (v !== undefined) module.exports = v;
    }
    else if (typeof define === "function" && define.amd) {
        define(["require", "exports", "./controls", "./templates", "./tools", "./page"], factory);
    }
})(function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    const controls_1 = require("./controls");
    const templates_1 = require("./templates");
    const tools_1 = require("./tools");
    const page_1 = require("./page");
    class base {
        constructor(model, baseType) {
            this.model = model;
            this.baseType = baseType;
        }
        attachPivot(elem) {
            if (elem.up("[data-m-type='pivot']")) {
                const pivotName = elem.up("[data-m-type='pivot']").attribute("data-m-page");
                return controls_1.controls.getPivot(pivotName);
            }
            return undefined;
        }
        inject(type, method, func) {
            const self = this;
            if (func == null) {
                throw new Error("Error: No function passed for injection!");
            }
            switch (type.lower()) {
                case "append":
                    self[`${method}_m_inject`] = func;
                    break;
                case "overwrite":
                    self[method] = func;
                    break;
                default:
                    throw new Error("Error: Invalid injection type!");
            }
            return self;
        }
        on(method, func, overwrite = false) {
            const self = this;
            self.inject((overwrite) ? "overwrite" : "append", tools_1.tools.eventEnumToString(method), func);
            return self;
        }
        action(action, model, func) {
            const self = this;
            page_1.page.action(action, model, func);
            return self;
        }
        formatData(item, isTable = true) {
            const self = this;
            const t = (self._template != null) ? self._template : self._elem.innerHTML;
            return templates_1.templates.merge(t, item, isTable);
        }
        clearAlerts() {
            const self = this;
            page_1.page.clearAlerts(`[data-m-type='${self.baseType}'][data-m-model='${self.model}'] [data-m-segment='alert']`);
        }
        showAlerts(className, txt, jsn, xml) {
            const self = this;
            page_1.page.showAlerts(`[data-m-type='${self.baseType}'][data-m-model='${self.model}'] [data-m-segment='alert']`, className, txt, jsn, xml);
        }
        loadSelects(selects, callback, reload = false) {
            page_1.page.loadSelects(selects, callback, reload);
        }
        clearFilters(selector) {
            page_1.page.clearFilters(selector);
        }
        shouldRoute(options) {
            if (options == null || options.hasRouting == null || options.hasRouting) {
                return true;
            }
            return false;
        }
    }
    exports.base = base;
});
//# sourceMappingURL=base.js.map