(function (factory) {
    if (typeof module === "object" && typeof module.exports === "object") {
        var v = factory(require, exports);
        if (v !== undefined) module.exports = v;
    }
    else if (typeof define === "function" && define.amd) {
        define(["require", "exports", "./schema", "./routing"], factory);
    }
})(function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    const schema_1 = require("./schema");
    const routing_1 = require("./routing");
    var controls;
    (function (controls) {
        function polyfill() {
            function fillDateInput(currentDate, keyCode) {
                const currentLength = currentDate.length;
                const lastCharEntered = currentDate[currentLength - 1];
                const daysInMonth = ["0", "31", "28", "31", "30", "31", "30", "31", "31", "30", "31", "30", "31"];
                const currentMonth = currentLength > 2 ? Number(currentDate.substring(0, 2)) : 0;
                if (currentLength == 2 && currentDate[0] == "1" && lastCharEntered == "/")
                    return "01/";
                if (!isNumber(lastCharEntered)) {
                    return currentDate.substring(0, currentLength - 1);
                }
                if (currentLength > 10) {
                    return currentDate.substring(0, 10);
                }
                if (currentLength == 1 && Number(currentDate) > 1) {
                    return "0" + currentDate + '/';
                }
                else if (currentLength == 3) {
                    if (Number(lastCharEntered) > 3) {
                        return currentDate.substring(0, 2) + "/0" + lastCharEntered + '/';
                    }
                    else {
                        return currentDate.substring(0, 2) + "/" + lastCharEntered;
                    }
                }
                else if (currentLength == 4 && Number(currentDate[3]) > 3) {
                    return currentDate.substring(0, 3) + "0" + currentDate[3] + '/';
                }
                else if (currentLength == 4 && Number(currentDate[3]) == 3 && currentMonth == 2) {
                    return currentDate.substring(0, 3) + "0" + currentDate[3] + '/';
                }
                else if (currentLength == 4 && Number(currentDate[3]) == 3 && currentMonth != 0) {
                    if (keyCode != 8) {
                        if (daysInMonth[currentMonth] == "30") {
                            return currentDate.substring(0, 3) + daysInMonth[currentMonth] + '/';
                        }
                        else {
                            return currentDate;
                        }
                    }
                    else {
                        return currentDate.substring(0, 3);
                    }
                }
                else if (currentLength == 2) {
                    if (keyCode != 8) {
                        if (Number(lastCharEntered) > 2 && currentDate[0] == "1") {
                            return currentDate.substring(0, currentLength - 1);
                        }
                        else {
                            return currentDate + '/';
                        }
                    }
                }
                else if (currentLength == 5) {
                    if (keyCode != 8) {
                        if (Number(currentDate[3]) == 3 && Number(lastCharEntered) > 1) {
                            return currentDate.substring(0, currentLength - 1);
                        }
                        else {
                            return currentDate + '/';
                        }
                    }
                }
                else if (currentLength == 6) {
                    return currentDate.substring(0, 5) + "/" + lastCharEntered;
                }
                else if (currentLength == 10 && Number(currentDate[3]) == 2 && Number(currentDate[4]) == 9 && currentMonth == 2) {
                    if (!isLeapYear(currentDate.substring(6))) {
                        return currentDate.replace("29", "28");
                    }
                }
                return currentDate;
            }
            function fillTimeInput(currentTime, keyCode) {
                const currentLength = currentTime.length;
                const lastCharEntered = currentTime[currentLength - 1];
                if (currentLength < 6 && !isNumber(lastCharEntered)) {
                    return currentTime.substring(0, currentLength - 1);
                }
                if ((currentLength == 6 || currentLength == 7) && !(lastCharEntered.toLowerCase() == "a" || lastCharEntered.toLowerCase() == "p")) {
                    return currentTime.substring(0, currentLength - 1);
                }
                if (currentLength > 8) {
                    return currentTime.substring(0, 8);
                }
                if (currentLength == 1 && Number(currentTime) > 1) {
                    return "0" + currentTime + ':';
                }
                else if (currentLength == 3) {
                    if (Number(lastCharEntered) > 5) {
                        return currentTime.substring(0, 2) + ":0" + lastCharEntered + ' ';
                    }
                    else {
                        return currentTime.substring(0, 2) + ":" + lastCharEntered;
                    }
                }
                else if (currentLength == 4 && Number(currentTime[3]) > 5) {
                    return currentTime.substring(0, 3) + "0" + currentTime[3] + ' ';
                }
                else if (currentLength == 2) {
                    if (keyCode != 8) {
                        return currentTime + ':';
                    }
                }
                else if (currentLength == 5) {
                    if (keyCode != 8) {
                        return currentTime + ' ';
                    }
                }
                else if (currentLength == 6) {
                    if (lastCharEntered.toLowerCase() == "a")
                        return currentTime.substring(0, 5) + " AM";
                    if (lastCharEntered.toLowerCase() == "p")
                        return currentTime.substring(0, 5) + " PM";
                }
                else if (currentLength == 6 || currentLength == 7) {
                    if (keyCode == 8) {
                        return currentTime.substring(0, currentLength - 1);
                    }
                    else {
                        if (lastCharEntered.toLowerCase() == "a")
                            return currentTime.substring(0, 5) + " AM";
                        if (lastCharEntered.toLowerCase() == "p")
                            return currentTime.substring(0, 5) + " PM";
                    }
                }
                return currentTime;
            }
            function isNumber(n) {
                return !isNaN(parseFloat(n)) && isFinite(n);
            }
            function isLeapYear(year) {
                return ((year % 4 == 0) && (year % 100 != 0)) || (year % 400 == 0);
            }
            function checkDateInputSupport() {
                const input = document.createElement("input");
                input.setAttribute("type", "date");
                const notADateValue = "not-a-date";
                input.setAttribute("value", notADateValue);
                return !(input.value === notADateValue);
            }
            if (!checkDateInputSupport() && typeof window.orientation === 'undefined') {
                schema_1.m.globals.requiresDateTimePolyfill = true;
                document.querySelectorAll("input[type=date]").each((idx, elem) => {
                    elem.placeholder = "mm/dd/yyyy";
                    elem.addEventListener("keyup", (e) => {
                        elem.val(fillDateInput(elem.val(), e.keyCode));
                    });
                });
                document.querySelectorAll("input[type=time]").each((idx, elem) => {
                    elem.placeholder = "--:-- --";
                    elem.addEventListener("keyup", (e) => {
                        elem.value = fillTimeInput(elem.val(), e.keyCode);
                    });
                });
            }
        }
        controls.polyfill = polyfill;
        function getPivot(name, callback) {
            let p;
            if (schema_1.m.globals["pivots"][name] != null) {
                p = schema_1.m.globals["pivots"][name];
            }
            else {
                p = new controls.pivot(document.querySelector(`[data-m-type='pivot'][data-m-page='${name}']`));
                schema_1.m.globals["pivots"][name] = p;
            }
            if (callback != null) {
                callback(p);
            }
            return p;
        }
        controls.getPivot = getPivot;
        class pivots {
            static bindAll(callback) {
                const pivots = document.querySelectorAll("[data-m-type='pivot']");
                for (let i = 0; i < pivots.length; i++) {
                    const section = pivots[i];
                    const page = section.attribute("data-m-page");
                    if (schema_1.m.globals["pivots"][page] == null) {
                        const p = new controls.pivot(section);
                        schema_1.m.globals["pivots"][page] = p;
                        section.querySelectorAll("[data-m-pivot]").each((idx, elem) => {
                            if (elem.up("[data-m-type='pivot']").attribute("data-m-page") == null || elem.up("[data-m-type='pivot']").attribute("data-m-page") === page) {
                                elem.addEvent("click", (e) => {
                                    e.preventDefault();
                                    const p1 = schema_1.m.globals["pivots"][page];
                                    p.exact(elem.attribute("data-m-pivot"));
                                }, true);
                            }
                        });
                    }
                }
                if (callback != null) {
                    callback();
                }
            }
        }
        controls.pivots = pivots;
        class pivot {
            constructor(pivotCollection, displayIndex = 0, nextButton, previousButton) {
                this.pivotCollection = pivotCollection;
                this.displayIndex = displayIndex;
                this.nextButton = nextButton;
                this.previousButton = previousButton;
                this._items = [];
                this._preEventFunctions = {};
                this._postEventFunctions = {};
                const self = this;
                self._name = pivotCollection.attribute("data-m-page");
                self._pivotContainer = pivotCollection;
                self._nextButton = (nextButton != null) ? document.querySelector(`#${nextButton}`) : self._pivotContainer.querySelector("[data-m-segment='controls'] [data-m-action='next']");
                self._previousButton = (previousButton != null) ? document.querySelector(`#${previousButton}`) : self._pivotContainer.querySelector("[data-m-segment='controls'] [data-m-action='previous']");
                if (self.displayIndex != null) {
                    const itemList = self._pivotContainer.querySelectorAll("[data-m-segment='pivot-item']");
                    itemList.each(function (idx, elem) {
                        if (elem.up("[data-m-type='pivot']").attribute("data-m-page") == null || elem.up("[data-m-type='pivot']").attribute("data-m-page") === self._name) {
                            self._items.push(elem);
                            if (idx == self.displayIndex) {
                                self.init(elem);
                                elem.show();
                                const hashName = (routing_1.routing.getRouteName() != null && routing_1.routing.getRouteName() != "") ? document.location.hash : `#/${elem.attribute("data-m-page")}/`;
                                history.replaceState(null, null, hashName);
                            }
                            else {
                                elem.hide();
                            }
                        }
                    });
                }
            }
            init(el) {
                const self = this;
                self._item = {
                    parent: el.parentElement,
                    current: el,
                    next: (el.nextElementSibling != null && el.nextElementSibling.attribute("data-m-segment") != null && el.nextElementSibling.attribute("data-m-segment") == "pivot-item") ? el.nextElementSibling : null,
                    previous: (el.previousElementSibling != null && el.previousElementSibling.attribute("data-m-segment") != null && el.previousElementSibling.attribute("data-m-segment") == "pivot-item") ? el.previousElementSibling : null,
                };
                if (self._nextButton != null || self._previousButton != null) {
                    if (self._item.next == null) {
                        self._nextButton.attribute("disabled", "disabled");
                    }
                    else {
                        self._nextButton.removeAttribute("disabled");
                        self._nextButton.addEvent("click", function (e) {
                            e.preventDefault();
                            self.next();
                        }, true);
                    }
                    if (self._item.previous == null) {
                        self._previousButton.attribute("disabled", "disabled");
                    }
                    else {
                        self._previousButton.removeAttribute("disabled");
                        self._previousButton.addEvent("click", function (e) {
                            e.preventDefault();
                            self.previous();
                        }, true);
                    }
                }
                if (self._item.current.attribute('data-m-page') != null) {
                    if (self._postEventFunctions[self._item.current.attribute('data-m-page')] != undefined) {
                        self._postEventFunctions[self._item.current.attribute('data-m-page')]();
                    }
                }
            }
            applyPreEvent(el) {
                const self = this;
                if (el.attribute('data-m-page') != null) {
                    if (self._preEventFunctions[el.attribute('data-m-page')] != undefined) {
                        self._preEventFunctions[el.attribute('data-m-page')]();
                    }
                }
            }
            addPreEvent(name, func) {
                const self = this;
                if (self._preEventFunctions[name] == undefined) {
                    self._preEventFunctions[name] = func;
                }
                else {
                    console.log(`${name} pre-event function already exists.`);
                }
            }
            addPostEvent(name, func) {
                const self = this;
                if (self._postEventFunctions[name] == undefined) {
                    self._postEventFunctions[name] = func;
                }
                else {
                    console.log(`${name} pre-event function already exists.`);
                }
            }
            removePostEvent(name) {
                delete this._postEventFunctions[name];
            }
            removePreEvent(name) {
                delete this._preEventFunctions[name];
            }
            next(callback) {
                const self = this;
                if (!self._item.next) {
                    console.log("Couldn't find next pivot. Using last pivot instead.");
                    self._item.next = self._items[self._items.length - 1];
                }
                self.applyPreEvent(self._item.current);
                for (let i = 0; i < self._items.length; i++) {
                    self._items[i].hide();
                }
                self._item.next.show();
                if (callback != null) {
                    callback(self._item.next.attribute("data-m-page"));
                }
                self.init(self._item.next);
            }
            previous(callback) {
                const self = this;
                if (!self._item.previous) {
                    console.log("Couldn't find previous pivot. Using first pivot instead.");
                    self._item.previous = self._items[0];
                }
                self.applyPreEvent(self._item.current);
                for (let i = 0; i < self._items.length; i++) {
                    self._items[i].hide();
                }
                self._item.previous.show();
                if (callback != null) {
                    callback(self._item.previous.attribute("data-m-page"));
                }
                self.init(self._item.previous);
            }
            exact(target) {
                const self = this;
                let idx;
                if (isNaN(target)) {
                    for (let i = 0; i < self._items.length; i++) {
                        const page = self._items[i].attribute("data-m-page");
                        if (page != null && page == target) {
                            idx = i;
                        }
                    }
                    if (idx == null) {
                        throw new Error(`Error: Cannot find a pivot with page name ${target}`);
                    }
                }
                else {
                    idx = target;
                }
                if (!self._items[idx]) {
                    console.log(`Error: No pivot at index ${idx}`);
                    return false;
                }
                self.applyPreEvent(self._item.current);
                for (let i = 0; i < self._items.length; i++) {
                    self._items[i].hide();
                }
                self._items[idx].show();
                if (self._items[idx].attribute("data-m-page") != null && schema_1.m.globals.hashLoadedFromApplication) {
                    routing_1.routing.setRouteUrl(self._items[idx].attribute("data-m-page"), "", true);
                }
                self.init(self._items[idx]);
                return true;
            }
        }
        controls.pivot = pivot;
    })(controls = exports.controls || (exports.controls = {}));
});
//# sourceMappingURL=controls.js.map