(function (factory) {
    if (typeof module === "object" && typeof module.exports === "object") {
        var v = factory(require, exports);
        if (v !== undefined) module.exports = v;
    }
    else if (typeof define === "function" && define.amd) {
        define(["require", "exports", "./base", "./schema", "./tools", "./routing", "./web"], factory);
    }
})(function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    const base_1 = require("./base");
    const schema_1 = require("./schema");
    const tools_1 = require("./tools");
    const routing_1 = require("./routing");
    const web_1 = require("./web");
    class forms {
        static bindAll(callback) {
            let sections = document.querySelectorAll("[data-m-type='form']");
            for (let i = 0; i < sections.length; i++) {
                let section = sections[i];
                if (section.attribute("data-m-autoload") == null || section.attribute("data-m-autoload") == "true") {
                    let model = section.attribute("data-m-model");
                    let mID = section.attribute("id");
                    let gTypeName = (mID != null) ? `${mID}_${model}` : model;
                    if (schema_1.m.globals["forms"][gTypeName] == null) {
                        let f = new form(model);
                        f.id = mID;
                        f.gTypeName = gTypeName;
                        f.init();
                    }
                }
            }
            if (callback != null) {
                callback();
            }
        }
    }
    exports.forms = forms;
    class form extends base_1.base {
        constructor(model, options) {
            super(model, schema_1.FORM);
            this.model = model;
            this.options = options;
            this._defaults = (schema_1.m.config["config.forms.defaults"] != null) ? schema_1.m.config["config.forms.defaults"] : [];
            this.hasLoaded = false;
            var self = this;
            self.id = (options != null) ? options.id : null;
            self.gTypeName = (options != null && options.id != null) ? `${options.id}_${model}` : model;
            schema_1.m.globals["forms"][self.gTypeName] = self;
            self._elem = (self.id != null) ? document.querySelector(`#${self.id}`) : document.querySelector(`[data-m-type='form'][data-m-model='${self.model}']`);
        }
        loadDefaults() {
            var self = this;
            if (self._defaults != null && self._defaults.length > 0) {
                for (let i = 0; i < self._defaults.length; i++) {
                    let pair = self._defaults[i];
                    for (let k in pair) {
                        if (pair.hasOwnProperty(k)) {
                            let v = pair[k];
                            if (k.contains(`${self.model}_`) && document.querySelector(`#${k}`) != null) {
                                document.querySelector(`#${k}`).val(v);
                            }
                            else if (!k.contains("_") && document.querySelector(`#${self.model}_${k}`) != null) {
                                document.querySelector(`#${self.model}_${k}`).val(v);
                            }
                        }
                    }
                }
            }
        }
        init() {
            var self = this;
            self.hasLoaded = true;
            if (self._elem != null) {
                self.pivot = self.attachPivot(self._elem);
                self._name = self._elem.attribute("data-m-page");
                let selects = self._elem.querySelectorAll("select");
                self.loadSelects(selects, () => {
                    let parameters;
                    let defaults;
                    let qs = web_1.web.querystring();
                    if (qs != "") {
                        defaults = tools_1.tools.formatOptions(qs, schema_1.OptionTypes.QUERYSTRING);
                    }
                    if (schema_1.m.globals.firstLoad) {
                        parameters = routing_1.routing.getRouteUrl({ "PageSize": 1, "PageIndex": 1, "_SortOrder": 1, "_SortDirection": 1 });
                    }
                    self.loadForm(parameters, defaults);
                });
                let controlBlocks = self._elem.querySelectorAll("[data-m-segment='controls']");
                controlBlocks.each((idx, elem) => {
                    let actions = elem.querySelectorAll("[data-m-action]");
                    actions.each((indx, el) => {
                        switch (el.attribute("data-m-action").lower()) {
                            case "save":
                                el.addEvent("click", function (e) {
                                    e.preventDefault();
                                    el.attribute("disabled", "disabled");
                                    if (schema_1.m.globals.actions != null && schema_1.m.globals.actions[`${self.model.lower()}_${el.attribute("data-m-action").lower()}`] != null) {
                                        schema_1.m.globals.actions[`${self.model.lower()}_${el.attribute("data-m-action").lower()}`]();
                                    }
                                    else {
                                        if (self.saving_m_inject != null) {
                                            self.saving_m_inject();
                                        }
                                        if (self.isValid()) {
                                            let parameters = {};
                                            let hasPrimary = false;
                                            self.elem.querySelectorAll("input, select, textarea").each(function (idx, elem) {
                                                parameters[elem.attribute("name")] = elem.val();
                                                if (elem.attribute("data-m-primary") != null && elem.attribute("data-m-primary").toBool() && elem.val() != "") {
                                                    hasPrimary = true;
                                                }
                                            });
                                            if (!hasPrimary) {
                                                web_1.web.post(`${routing_1.routing.getAPIURL(self.model)}`, parameters, null, "json", (data) => {
                                                    self.save(data, el.attribute("data-m-pivot"), el);
                                                }, (txt, jsn, xml) => {
                                                    self.showAlerts(schema_1.DANGER, txt, jsn, xml);
                                                    el.removeAttribute("disabled");
                                                });
                                            }
                                            else {
                                                web_1.web.put(`${routing_1.routing.getAPIURL(self.model)}`, parameters, null, "json", (data) => {
                                                    self.save(data, el.attribute("data-m-pivot"), el);
                                                }, (txt, jsn, xml) => {
                                                    self.showAlerts(schema_1.DANGER, txt, jsn, xml);
                                                    el.removeAttribute("disabled");
                                                });
                                            }
                                        }
                                        else {
                                            el.removeAttribute("disabled");
                                        }
                                    }
                                }, true);
                                break;
                            case "cancel":
                                el.addEvent("click", function (e) {
                                    e.preventDefault();
                                    if (schema_1.m.globals.actions != null && schema_1.m.globals.actions[`${self.model.lower()}_${el.attribute("data-m-action").lower()}`] != null) {
                                        schema_1.m.globals.actions[`${self.model.lower()}_${el.attribute("data-m-action").lower()}`]();
                                    }
                                    else {
                                        self.clearForm();
                                        if (self.pivot != null) {
                                            (el.attribute("data-m-pivot") != null)
                                                ? self.pivot.exact(el.attribute("data-m-pivot"))
                                                : self.pivot.previous((previousPage) => {
                                                    if (previousPage != null) {
                                                        routing_1.routing.setRouteUrl(previousPage, "", true);
                                                    }
                                                });
                                        }
                                        if (schema_1.m.globals["lists"][self.model] != null) {
                                            schema_1.m.globals["lists"][self.model].callListing();
                                        }
                                        if (self.cancel_m_inject != null) {
                                            self.cancel_m_inject();
                                        }
                                    }
                                }, true);
                                break;
                            default:
                                if (schema_1.m.globals.actions != null && schema_1.m.globals.actions[`${self.model.lower()}_${el.attribute("data-m-action").lower()}`] != null) {
                                    el.addEvent("click", function (e) {
                                        e.preventDefault();
                                        schema_1.m.globals.actions[`${self.model.lower()}_${el.attribute("data-m-action").lower()}`]();
                                    }, true);
                                }
                                break;
                        }
                    });
                });
                if (self.init_m_inject != null) {
                    self.init_m_inject();
                }
            }
            return self;
        }
        save(data, pivotPosition, saveElement) {
            var self = this;
            self.elem.querySelectorAll("[data-m-primary]").each((idx, elem) => {
                elem.val(data[elem.attribute("name")]);
            });
            if (self.pivot != null) {
                (pivotPosition != null)
                    ? self.pivot.exact(pivotPosition)
                    : self.pivot.previous((previousPage) => {
                        if (previousPage != null) {
                            routing_1.routing.setRouteUrl(previousPage, "", true);
                        }
                    });
            }
            if (schema_1.m.globals["lists"][self.model] != null) {
                try {
                    schema_1.m.globals["lists"][self.model].callListing();
                }
                catch (e) { }
            }
            saveElement.removeAttribute("disabled");
            if (self.save_m_inject != null) {
                self.save_m_inject(data);
            }
        }
        loadForm(parameters, defaults) {
            var self = this;
            self.clearForm();
            if (defaults != null) {
                for (let prop in defaults) {
                    if (defaults.hasOwnProperty(prop) && defaults[prop] != null && document.querySelector(`#${self.model}_${prop}`) != null) {
                        document.querySelector(`#${self.model}_${prop}`).val(defaults[prop]);
                    }
                }
            }
            if (parameters != null && Object.keys(parameters).length > 0) {
                web_1.web.get(`${routing_1.routing.getAPIURL(self.model)}${web_1.web.querystringify(parameters)}`, parameters, null, "json", function (data) {
                    if (data instanceof Array) {
                        data = data[0];
                    }
                    for (let prop in data) {
                        if (self.id == null) {
                            if (data.hasOwnProperty(prop) && data[prop] != null && document.querySelector(`#${self.model}_${prop}`) != null) {
                                document.querySelector(`#${self.model}_${prop}`).val(data[prop]);
                            }
                        }
                        else {
                            if (data.hasOwnProperty(prop) && data[prop] != null && document.querySelector(`#${self.id} > fieldset > #${self.model}_${prop}`) != null) {
                                document.querySelector(`#${self.id} > fieldset > #${self.model}_${prop}`).val(data[prop]);
                            }
                        }
                    }
                    if (self.loadForm_m_inject != null) {
                        self.loadForm_m_inject(data);
                    }
                });
            }
            else if (self.loadForm_m_inject != null) {
                self.loadForm_m_inject(null);
            }
        }
        clearForm(selector, callback) {
            var self = this;
            var f = (self._elem != null) ? self._elem : document.querySelector(selector);
            document.querySelectorAll(".error").each(function (idx, elem) {
                elem.removeClass("error");
            });
            document.querySelectorAll(".label-error").each(function (idx, elem) {
                elem.removeClass("label-error");
            });
            f.querySelectorAll("input:not([data-m-ignore='true']), select:not([data-m-ignore='true'])").each(function (idx, elem) {
                elem.val("");
            });
            f.querySelectorAll("textarea:not([data-m-ignore='true'])").each(function (idx, elem) {
                elem.val("");
            });
            f.querySelectorAll("input[type='checkbox']:not([data-m-ignore='true'])").each(function (idx, elem) {
                elem.val("");
            });
            self.clearAlerts();
            self.loadDefaults();
            if (callback != null) {
                callback();
            }
            if (self.clearForm_m_inject != null) {
                self.clearForm_m_inject();
            }
        }
        isValid(selector) {
            var self = this;
            self.clearAlerts();
            var result = "";
            var f = (self._elem != null) ? self._elem : document.querySelector(selector);
            f.querySelectorAll(".error").each(function (idx, elem) {
                elem.removeClass("error");
            });
            f.querySelectorAll(".label-error").each(function (idx, elem) {
                elem.removeClass("label-error");
            });
            var isValid = true;
            var required = f.querySelectorAll("[required='required']");
            required.each(function (idx, elem) {
                try {
                    if (elem.val() == null || elem.val().trim() === "") {
                        isValid = false;
                        elem.addClass("error");
                        if (f.querySelector(`label[for='${elem.attribute("id")}']`) != null) {
                            f.querySelector(`label[for='${elem.attribute("id")}']`).addClass("label-error");
                        }
                        if (f.querySelector(`label[for='${elem.attribute("id")}']`) != null && f.querySelector(`label[for='${elem.attribute("id")}']`).innerText != "") {
                            result += `<p>${f.querySelector(`label[for='${elem.attribute("id")}']`).innerText} is a required field.</p>`;
                        }
                        else {
                            result += `<p>[${elem.attribute("name")}] is a required field.</p>`;
                        }
                    }
                }
                catch (e) {
                    console.log(`An error occurred while validating the form: ${e}`);
                    isValid = false;
                }
            });
            if (!isValid) {
                self.showAlerts(schema_1.DANGER, result);
                window.scrollTo(0, 0);
            }
            return isValid;
        }
        get elem() {
            return this._elem;
        }
        set elem(f) {
            this._elem = f;
        }
    }
    exports.form = form;
});
//# sourceMappingURL=forms.js.map