(function (factory) {
    if (typeof module === "object" && typeof module.exports === "object") {
        var v = factory(require, exports);
        if (v !== undefined) module.exports = v;
    }
    else if (typeof define === "function" && define.amd) {
        define(["require", "exports", "./templates", "./tools", "./schema", "./storage", "./controls", "./routing", "./web", "./lists", "./forms"], factory);
    }
})(function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    const templates_1 = require("./templates");
    const tools_1 = require("./tools");
    const schema_1 = require("./schema");
    const storage_1 = require("./storage");
    const controls_1 = require("./controls");
    const routing_1 = require("./routing");
    const web_1 = require("./web");
    const lists_1 = require("./lists");
    const forms_1 = require("./forms");
    function onready(callback, appName) {
        document.addEventListener("DOMContentLoaded", function (e) {
            templates_1.templates.master.loadMaster(document.documentElement.outerHTML).then(() => {
                let proms = [];
                document.querySelectorAll("[data-m-include]").each((idx, elem) => {
                    let prom = templates_1.templates.load(elem.attribute("data-m-include")).then(result => {
                        if (elem.attribute("data-m-type") != null && elem.attribute("data-m-type") == "markdown") {
                            elem.innerHTML = templates_1.templates.markdown.toHTML(result);
                            elem.show();
                        }
                        else {
                            elem.innerHTML = result;
                        }
                    });
                    proms.push(prom);
                });
                Promise.all(proms).then(() => {
                    document.querySelectorAll("[data-m-type='markdown']").each((idx, elem) => {
                        if (elem.attribute("data-m-include") == null) {
                            elem.innerHTML = templates_1.templates.markdown.toHTML(elem.innerHTML);
                            elem.show();
                        }
                    });
                    let root = routing_1.routing.getApplicationRoot(document.documentElement.outerHTML);
                    appName = (appName != null) ? appName : routing_1.routing.getApplicationName(document.documentElement.outerHTML);
                    let iDB = (appName == null) ? schema_1.DB : `${schema_1.DB}.${appName.lower()}`;
                    let iDBStore = (appName == null) ? schema_1.STORE : `${schema_1.STORE}.${appName.lower()}`;
                    let storage = new storage_1.store(iDB, schema_1.DBVERSION, iDBStore);
                    storage.init().then((result) => {
                        return storage.getItem("metron.config", "value");
                    }).then((result) => {
                        if (result != null) {
                            schema_1.m.config = JSON.parse(result);
                            schema_1.m.globals.firstLoad = true;
                            if (callback != null) {
                                callback(e);
                            }
                        }
                        else {
                            new Promise((resolve, reject) => {
                                web_1.web.loadJSON(`${root}/metron.json`, (configData) => {
                                    for (let obj in configData) {
                                        if (schema_1.m.config[obj] == null) {
                                            schema_1.m.config[obj] = configData[obj];
                                        }
                                    }
                                    schema_1.m.config["config.baseURL"] = `${document.location.protocol}//${document.location.host}`;
                                    storage.init().then((result) => {
                                        return storage.setItem("metron.config", JSON.stringify(schema_1.m.config));
                                    }).then((result) => {
                                        resolve(configData);
                                    }).catch((rs) => {
                                        console.log(`Error: Failed to access storage. ${rs}`);
                                    });
                                });
                            }).then(() => {
                                schema_1.m.globals.firstLoad = true;
                                if (callback != null) {
                                    callback(e);
                                }
                            }).catch((rsn) => {
                                console.log(`Error: Promise execution failed! ${rsn}`);
                            });
                        }
                    }).catch((reason) => {
                        console.log(`Error: Failed to access storage. ${reason}`);
                    });
                });
            }).catch(() => {
                console.log("Failed to check for master page.");
            });
        });
    }
    exports.onready = onready;
    function load(re, func) {
        let n, f;
        if (typeof func == "object") {
            n = func.n;
            f = func.func;
        }
        else {
            f = func;
        }
        let h = () => {
            if (n !== undefined) {
                let p = document.querySelector("[data-m-type='pivot']");
                if (p !== undefined) {
                    controls_1.controls.getPivot(p.attribute("data-m-page")).exact(n);
                }
            }
            f();
        };
        routing_1.routing.add(re, h);
    }
    exports.load = load;
    function ifQuerystring(callback) {
        let qs = web_1.web.querystring();
        if (qs != "") {
            let parameters = tools_1.tools.formatOptions(qs, schema_1.OptionTypes.QUERYSTRING);
            if (callback != null) {
                callback(parameters);
            }
        }
    }
    exports.ifQuerystring = ifQuerystring;
    function loadOptionalFunctionality() {
        if (typeof Awesomplete !== undefined) {
            document.querySelectorAll("[data-m-autocomplete]").each((idx, elem) => {
                let endpoint = elem.attribute("data-m-autocomplete");
                let url = (endpoint.toLowerCase().startsWith("http")) ? endpoint : routing_1.routing.getAPIURL(endpoint);
                let awesome = new Awesomplete(elem, {
                    minChars: 1,
                    sort: false,
                    maxItems: 15,
                    replace: function (item) {
                        if (elem.attribute("data-m-search-hidden-store") != null && elem.attribute("data-m-search-hidden-store") != '') {
                            this.input.value = item.label;
                            document.querySelector(`#${elem.attribute("data-m-search-hidden-store")}`).val(item.value);
                            document.querySelector(`#${elem.attribute("data-m-search-hidden-store")}`).dispatchEvent(new Event("change"));
                        }
                        else {
                            this.input.value = item.value;
                        }
                    }
                });
                elem.removeEvent("keyup").addEvent("keyup", function (e) {
                    web_1.web.get(`${url}${web_1.web.querystringify({ IsActive: true, Search: this.val() })}`, null, null, "json", (result) => {
                        let list = [];
                        if (result != null) {
                            for (var a in result) {
                                if (result.hasOwnProperty(a)) {
                                    if (result[a][elem.attribute("data-m-search-text")] != null) {
                                        var item = { label: result[a][elem.attribute("data-m-search-text")], value: result[a][elem.attribute("data-m-search-value")] };
                                        list.push(item);
                                    }
                                }
                            }
                            awesome.list = list;
                        }
                    });
                });
            });
        }
    }
    exports.loadOptionalFunctionality = loadOptionalFunctionality;
    window.onhashchange = function () {
        if (!schema_1.m.globals.hashLoadedFromApplication) {
            let hasPivoted = false;
            let section = document.querySelector("[data-m-type='pivot']");
            if (section != null) {
                let page = section.attribute("data-m-page");
                if (page != null) {
                    let p = controls_1.controls.getPivot(page);
                    p.previous();
                    hasPivoted = true;
                }
            }
            if (!hasPivoted) {
                window.location.reload(false);
            }
        }
        schema_1.m.globals.hashLoadedFromApplication = false;
    };
    onready((e) => {
        function recursePivot(elem) {
            if (elem != null) {
                elem.show();
                let route = elem.attribute("data-m-page");
                let pivot = elem.up("[data-m-type='pivot']");
                let pivotPageName = pivot.attribute("data-m-page");
                elem.up("[data-m-type='pivot']").querySelectorAll("[data-m-segment='pivot-item']").each((idx, el) => {
                    if (el.up("[data-m-type='pivot']").attribute("data-m-page") === pivotPageName) {
                        if (el.attribute("data-m-page") != route) {
                            el.hide();
                        }
                    }
                });
                let parent = elem.parent().up("[data-m-segment='pivot-item']");
                if (parent != null) {
                    recursePivot(parent);
                }
            }
        }
        let wantsAutoload = ((document.querySelector("[data-m-autoload]") != null) && (document.querySelector("[data-m-autoload]").attribute("data-m-autoload") == "true"));
        document.querySelectorAll("[data-m-state='hide']").each((idx, elem) => {
            elem.hide();
        });
        controls_1.controls.pivots.bindAll(() => {
            let route = routing_1.routing.getRouteName();
            if (route != null) {
                let page = document.querySelector(`[data-m-segment='pivot-item'][data-m-page="${route}"]`);
                recursePivot(page);
            }
            loadOptionalFunctionality();
            if (wantsAutoload) {
                lists_1.lists.bindAll(() => {
                    forms_1.forms.bindAll(() => {
                        controls_1.controls.polyfill();
                    });
                });
            }
        });
    });
});
//# sourceMappingURL=framework.js.map