(function (factory) {
    if (typeof module === "object" && typeof module.exports === "object") {
        var v = factory(require, exports);
        if (v !== undefined) module.exports = v;
    }
    else if (typeof define === "function" && define.amd) {
        define(["require", "exports", "./schema", "./base", "./routing", "./web", "./tools", "./storage"], factory);
    }
})(function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    const schema_1 = require("./schema");
    const base_1 = require("./base");
    const routing_1 = require("./routing");
    const web_1 = require("./web");
    const tools_1 = require("./tools");
    const storage_1 = require("./storage");
    class lists {
        static bindAll(callback) {
            let sections = document.querySelectorAll("[data-m-type='list']");
            for (let i = 0; i < sections.length; i++) {
                let section = sections[i];
                if (section.attribute("data-m-autoload") == null || section.attribute("data-m-autoload") == "true") {
                    let model = section.attribute("data-m-model");
                    let mID = section.attribute("id");
                    let gTypeName = (mID != null) ? `${mID}_${model}` : model;
                    if (schema_1.m.globals["lists"][gTypeName] == null) {
                        let l = new list(model);
                        l.id = mID;
                        l.gTypeName = gTypeName;
                        l.init();
                    }
                }
            }
            if (callback != null) {
                callback();
            }
        }
    }
    exports.lists = lists;
    class list extends base_1.base {
        constructor(model, options) {
            super(model, schema_1.LIST);
            this.model = model;
            this.options = options;
            this._filters = {};
            this.recycleBin = [];
            this.currentPageIndex = 1;
            this.pageSize = (!isNaN(schema_1.m.config["config.options.pageSize"])) ? schema_1.m.config["config.options.pageSize"] : 10;
            this.totalPageSize = 0;
            this.totalCount = 0;
            this.sortOrder = (schema_1.m.config["config.options.sortOrder"] != null) ? schema_1.m.config["config.options.sortOrder"] : "DateCreated";
            this.sortDirection = (schema_1.m.config["config.options.sortDirection"] != null) ? schema_1.m.config["config.options.sortDirection"] : "DESC";
            this._defaults = (schema_1.m.config["config.lists.defaults"] != null) ? schema_1.m.config["config.lists.defaults"] : {};
            var self = this;
            self.id = (options != null) ? options.id : null;
            self.gTypeName = (options != null && options.id != null) ? `${options.id}_${model}` : model;
            schema_1.m.globals["lists"][self.gTypeName] = self;
            self.setFilters();
        }
        init() {
            var self = this;
            self._elem = (self.id != null) ? document.querySelector(`#${self.id}`) : document.querySelector(`[data-m-type='list'][data-m-model='${self.model}']`);
            if (self._elem != null) {
                self.pivot = self.attachPivot(self._elem);
                self._name = self._elem.attribute("data-m-page");
                let filterBlocks = self._elem.querySelectorAll("[data-m-segment='filters']");
                filterBlocks.each(function (idx, elem) {
                    let filters = elem.querySelectorAll("[data-m-action='filter']");
                    self.loadFilters(filters);
                });
                var controlBlocks = self._elem.querySelectorAll("[data-m-segment='controls']");
                controlBlocks.each(function (idx, elem) {
                    let actions = elem.querySelectorAll("[data-m-action]");
                    actions.each(function (indx, el) {
                        switch (el.attribute("data-m-action").lower()) {
                            case "new":
                                el.addEvent("click", function (e) {
                                    e.preventDefault();
                                    if (schema_1.m.globals.actions != null && schema_1.m.globals.actions[`${self.model.lower()}_${el.attribute("data-m-action").lower()}`] != null) {
                                        schema_1.m.globals.actions[`${self.model.lower()}_${el.attribute("data-m-action").lower()}`]();
                                    }
                                    else {
                                        if (self.pivot != null) {
                                            (el.attribute("data-m-pivot") != null)
                                                ? self.pivot.exact(el.attribute("data-m-pivot"))
                                                : self.pivot.next((nextPage) => {
                                                    if (nextPage != null) {
                                                        routing_1.routing.setRouteUrl(nextPage, "", true);
                                                    }
                                                });
                                        }
                                        if (schema_1.m.globals["forms"][self.model] != null) {
                                            try {
                                                schema_1.m.globals["forms"][self.model].loadForm();
                                            }
                                            catch (e) {
                                                console.log(e);
                                            }
                                        }
                                        if (self.new_m_inject != null) {
                                            self.new_m_inject();
                                        }
                                    }
                                }, true);
                                break;
                            case "undo":
                                el.addEvent("click", function (e) {
                                    e.preventDefault();
                                    if (schema_1.m.globals.actions != null && schema_1.m.globals.actions[`${self.model.lower()}_${el.attribute("data-m-action").lower()}`] != null) {
                                        schema_1.m.globals.actions[`${self.model.lower()}_${el.attribute("data-m-action").lower()}`]();
                                    }
                                    else {
                                        self.undoLast();
                                    }
                                    if (self.undo_m_inject != null) {
                                        self.undo_m_inject();
                                    }
                                }, true);
                                el.hide();
                                break;
                            case "download":
                                el.addEvent("click", function (e) {
                                    e.preventDefault();
                                    if (schema_1.m.globals.actions != null && schema_1.m.globals.actions[`${self.model.lower()}_${el.attribute("data-m-action").lower()}`] != null) {
                                        schema_1.m.globals.actions[`${self.model.lower()}_${el.attribute("data-m-action").lower()}`]();
                                    }
                                    else {
                                        document.location.href = `${routing_1.routing.getAppUrl()}/${self.model}/download`;
                                    }
                                    if (self.download_m_inject != null) {
                                        self.download_m_inject();
                                    }
                                }, true);
                                break;
                            case "clear":
                                el.addEvent("click", function (e) {
                                    e.preventDefault();
                                    if (schema_1.m.globals.actions != null && schema_1.m.globals.actions[`${self.model.lower()}_${el.attribute("data-m-action").lower()}`] != null) {
                                        schema_1.m.globals.actions[`${self.model.lower()}_${el.attribute("data-m-action").lower()}`]();
                                    }
                                    else {
                                        self._elem.querySelectorAll("[data-m-segment='filters']").each((idx, elem) => {
                                            self.clearFilters(elem);
                                            for (let key in self.filters) {
                                                if (self._filters.hasOwnProperty(key)) {
                                                    self._filters[key] = null;
                                                }
                                            }
                                            const qs = web_1.web.querystring();
                                            if (qs != "") {
                                                self._filters = tools_1.tools.formatOptions(qs, schema_1.OptionTypes.QUERYSTRING);
                                            }
                                            self.currentPageIndex = 1;
                                            self.callListing();
                                        });
                                    }
                                    if (self.clear_m_inject != null) {
                                        self.clear_m_inject();
                                    }
                                }, true);
                                break;
                            default:
                                if (schema_1.m.globals.actions != null && schema_1.m.globals.actions[`${self.model.lower()}_${el.attribute("data-m-action").lower()}`] != null) {
                                    let ev;
                                    if (el.nodeName.lower() === "a" || el.nodeName.lower() === "button" || el.nodeName.lower() === "div" || el.nodeName.lower() === "span" || (el.nodeName.lower() === "input" && (el.attribute("type").lower() === "submit" || el.attribute("type").lower() === "button"))) {
                                        ev = "click";
                                    }
                                    else {
                                        ev = "change";
                                    }
                                    el.addEvent(ev, function (e) {
                                        e.preventDefault();
                                        schema_1.m.globals.actions[`${self.model.lower()}_${el.attribute("data-m-action").lower()}`]();
                                    });
                                }
                                break;
                        }
                    });
                });
                self.callListing();
                if (self.init_m_inject != null) {
                    self.init_m_inject();
                }
            }
            return self;
        }
        loadFilters(filters) {
            var self = this;
            var promises = [];
            filters.each(function (indx, el) {
                if (el.attribute("data-m-binding") != null) {
                    let binding = el.attribute("data-m-binding");
                    let key = (el.attribute("data-m-key")) != null ? el.attribute("data-m-key") : el.attribute("name");
                    if (!self._filters.hasOwnProperty(key)) {
                        console.log(`Warning: You have set the data-m-key to ${key}, which is not expected according to the <T> interface type of ${binding}.`);
                    }
                    let nm = el.attribute("name");
                    let nText = el.attribute("data-m-text");
                    let options = (el.attribute("data-m-options") != null) ? tools_1.tools.formatOptions(el.attribute("data-m-options")) : {};
                    let ajx = new Promise((resolve, reject) => {
                        web_1.web.get(`${routing_1.routing.getAPIURL(binding)}${self.withDefaults(options)}`, {}, null, "json", function (data) {
                            data.each((i, item) => {
                                if (self._filters[key] != null && self._filters[key] == item[key]) {
                                    el.append(`<option value="${item[key]}" selected="selected">${item[nText]}</option>`);
                                }
                                else {
                                    el.append(`<option value="${item[key]}">${item[nText]}</option>`);
                                }
                            });
                            resolve(data);
                        });
                    });
                    promises.push(ajx);
                    el.addEvent("change", function (e) {
                        let fil = self._filters;
                        fil[key] = (this.val() == '') ? null : this.val();
                        self._filters = fil;
                        self.currentPageIndex = 1;
                        self.callListing();
                    });
                }
                if (el.attribute("data-m-search") != null) {
                    el.addEvent("click", function (e) {
                        e.preventDefault();
                        let itm = this;
                        let fil = self._filters;
                        let terms = this.attribute("data-m-search").split(";");
                        terms.each(function (i, term) {
                            let parent = itm.parent();
                            fil[term.trim()] = (parent.querySelector(`#${itm.attribute("data-m-search-for")}`).val() == '') ? null : parent.querySelector(`#${itm.attribute("data-m-search-for")}`).val();
                        });
                        self._filters = fil;
                        self.currentPageIndex = 1;
                        self.callListing();
                    });
                }
            });
            Promise.all(promises).then(() => {
                if (self.loadFilters_m_inject != null) {
                    self.loadFilters_m_inject();
                }
            }).catch(function (reason) {
                console.log(`Error: Promise execution failed! ${reason}`);
            });
        }
        applyViewEvents() {
            var self = this;
            self._elem.querySelectorAll(`[data-m-action='edit']`).each(function (idx, elem) {
                elem.removeEvent("click").addEvent("click", function (e) {
                    e.preventDefault();
                    if (schema_1.m.globals.actions != null && schema_1.m.globals.actions[`${self.model.lower()}_${elem.attribute("data-m-action").lower()}`] != null) {
                        schema_1.m.globals.actions[`${self.model.lower()}_${elem.attribute("data-m-action").lower()}`](elem);
                    }
                    else {
                        let parameters = tools_1.tools.formatOptions(elem.attribute("data-m-primary"));
                        if (self.pivot != null) {
                            (elem.attribute("data-m-pivot") != null)
                                ? self.pivot.exact(elem.attribute("data-m-pivot"))
                                : self.pivot.next((nextPage) => {
                                    if (nextPage != null) {
                                        routing_1.routing.setRouteUrl(nextPage, web_1.web.querystringify(parameters), true);
                                    }
                                });
                        }
                        if (schema_1.m.globals["forms"][self.model] != null) {
                            try {
                                schema_1.m.globals["forms"][self.model].loadForm(parameters);
                            }
                            catch (e) { }
                        }
                        if (self.edit_m_inject != null) {
                            self.edit_m_inject();
                        }
                    }
                }, true);
            });
            self._elem.querySelectorAll(`[data-m-action='delete']`).each(function (idx, elem) {
                elem.removeEvent("click").addEvent("click", function (e) {
                    e.preventDefault();
                    if (schema_1.m.globals.actions != null && schema_1.m.globals.actions[`${self.model.lower()}_${elem.attribute("data-m-action").lower()}`] != null) {
                        schema_1.m.globals.actions[`${self.model.lower()}_${elem.attribute("data-m-action").lower()}`](elem);
                    }
                    else {
                        if (confirm('Are you sure you want to delete this record?')) {
                            let current = this;
                            let parameters = tools_1.tools.formatOptions(elem.attribute("data-m-primary"));
                            web_1.web.remove(`${routing_1.routing.getAPIURL(self.model)}${web_1.web.querystringify(parameters)}`, parameters, null, "json", function (data) {
                                if (data instanceof Array) {
                                    data = data[0];
                                }
                                self.recycleBin.push(data);
                                if (current.up("tr") != null) {
                                    current.up("tr").drop();
                                }
                                else {
                                    current.up(".row").drop();
                                }
                                if (self._elem.querySelector(`[data-m-action='undo']`)) {
                                    self._elem.querySelector(`[data-m-action='undo']`).show();
                                }
                                if (self.delete_m_inject != null) {
                                    self.delete_m_inject();
                                }
                            });
                        }
                    }
                }, true);
            });
            self._elem.querySelectorAll(`[data-m-action='sort']`).each(function (idx, elem) {
                elem.removeClass("pointer").addClass("pointer");
                elem.removeEvent("click").addEvent("click", function (e) {
                    self.sortOrder = elem.attribute("data-m-col");
                    if (self.sortDirection == "ASC") {
                        self.sortDirection = "DESC";
                    }
                    else {
                        self.sortDirection = "ASC";
                    }
                    self.callListing();
                }, true);
            });
            self._elem.querySelectorAll(`[data-m-action]`).each(function (idx, elem) {
                if (elem.attribute("data-m-action") != "edit" && elem.attribute("data-m-action") != "delete" && elem.attribute("data-m-action") != "sort" && elem.attribute("data-m-action") != "filter") {
                    if (schema_1.m.globals.actions != null && schema_1.m.globals.actions[`${self.model.lower()}_${elem.attribute("data-m-action").lower()}`] != null) {
                        let ev;
                        if (elem.nodeName.lower() === "a" || elem.nodeName.lower() === "button" || elem.nodeName.lower() === "div" || elem.nodeName.lower() === "span" || (elem.nodeName.lower() === "input" && (elem.attribute("type").lower() === "submit" || elem.attribute("type").lower() === "button"))) {
                            ev = "click";
                        }
                        else {
                            ev = "change";
                        }
                        elem.removeEvent(ev).addEvent(ev, function (e) {
                            e.preventDefault();
                            schema_1.m.globals.actions[`${self.model.lower()}_${elem.attribute("data-m-action").lower()}`](elem);
                        });
                    }
                }
            });
            if (self.applyViewEvents_m_inject != null) {
                self.applyViewEvents_m_inject();
            }
        }
        withDefaults(parameters) {
            var self = this;
            if (Object.keys(self._defaults).length) {
                let local = {};
                for (let k in self._defaults) {
                    if (self._defaults.hasOwnProperty(k)) {
                        local[k] = self._defaults[k];
                    }
                }
                let withDefaults = Object.extend(parameters, local);
                return web_1.web.querystringify(tools_1.tools.normalizeModelData(withDefaults));
            }
            return web_1.web.querystringify(tools_1.tools.normalizeModelData(parameters));
        }
        populateListing() {
            var self = this;
            try {
                self.clearTable(self._elem.querySelector(`[data-m-segment='list']`));
                self.populateTable(self._elem.querySelector(`[data-m-segment='list']`));
                self.totalCount = (self._items.length > 0) ? self._items[0]["TotalCount"] : 0;
                self.createPaging(self._elem.querySelector(`[data-m-segment='paging']`), self.totalCount);
                self.applyViewEvents();
                if (self.populateListing_m_inject != null) {
                    self.populateListing_m_inject();
                }
            }
            catch (e) {
                console.log(`Failed to populate listing: ${e}`);
            }
        }
        undoLast() {
            var self = this;
            web_1.web.post(`${routing_1.routing.getAPIURL(self.model)}/undo`, self.recycleBin.pop(), null, "json", function (data) {
                self.callListing();
            });
            if (self.recycleBin.length == 0) {
                self._elem.querySelector(`[data-m-action='undo']`).hide();
            }
            if (self.undoLast_m_inject != null) {
                self.undoLast_m_inject();
            }
        }
        callListing() {
            var self = this;
            self.clearAlerts();
            var parameters = Object.extend({ PageIndex: self.currentPageIndex, PageSize: self.pageSize, _SortOrder: self.sortOrder, _SortDirection: self.sortDirection }, self._filters);
            var url = (self.fetchURL != null) ? self.fetchURL : self.model;
            if (!self._elem.isHidden() && self.shouldRoute(self.options)) {
                routing_1.routing.setRouteUrl(self._name, web_1.web.querystringify(tools_1.tools.normalizeModelData(parameters)));
            }
            web_1.web.get(`${routing_1.routing.getAPIURL(url)}${self.withDefaults(parameters)}`, {}, null, "json", function (data) {
                self._items = data;
                self.populateListing();
                if (self.callListing_m_inject != null) {
                    self.callListing_m_inject(data);
                }
            }, (txt, jsn, xml) => {
                self.showAlerts(schema_1.DANGER, txt, jsn, xml);
            });
        }
        populateTable(selector) {
            var self = this;
            var tbody = selector.querySelector("[data-m-type='table-body']");
            var isTable = (tbody.nodeName.lower() == "tbody") ? true : false;
            self._items.each(function (idx, item) {
                selector.querySelector("[data-m-type='table-body']").append(self.formatData(item, isTable));
            });
            tbody.attribute("data-m-state", "show");
            if (isTable) {
                tbody.show("table-row-group");
            }
            else {
                tbody.show();
            }
        }
        clearTable(selector) {
            const self = this;
            if (String.isNullOrEmpty(self._template)) {
                const t = selector.up("[data-m-page]");
                try {
                    if (t != null) {
                        self._template = schema_1.m.globals.templates[t.attribute("data-m-page")];
                    }
                    if (self._template == null) {
                        self._template = selector.querySelector("[data-m-type='table-body'] [data-m-action='repeat']").outerHTML;
                        schema_1.m.globals.templates[t.attribute("data-m-page")] = self._template;
                    }
                }
                catch (e) {
                    console.log(`DOM has no element that matches the selector "${selector} [data-m-type='table-body'] [data-m-action='repeat']": ${e}`);
                }
            }
            selector.querySelector("[data-m-type='table-body']").empty();
        }
        getRows(selector) {
            return document.querySelectorAll(`${selector} [data-m-type='table-body'] [data-m-type='row']`).length;
        }
        setupPagingEvents(selector, filters) {
            var self = this;
            selector.querySelector("li > a[title='Previous']").removeEvent("click").addEvent("click", function (e) {
                e.preventDefault();
                self.pageListing(self.getPreviousPage(), filters);
            }, true);
            selector.querySelector("li > a[title='Next']").removeEvent("click").addEvent("click", function (e) {
                e.preventDefault();
                self.pageListing(self.getNextPage(), filters);
            }, true);
            selector.querySelector("li > a[title='First']").removeEvent("click").addEvent("click", function (e) {
                e.preventDefault();
                self.pageListing(1, filters);
            }, true);
            selector.querySelector("li > a[title='Last']").removeEvent("click").addEvent("click", function (e) {
                e.preventDefault();
                self.pageListing(self.totalPageSize, filters);
            }, true);
        }
        createPaging(selector, totalCount, filters) {
            var self = this;
            if (selector == null) {
                return;
            }
            if (self.currentPageIndex != null && self.pageSize != null) {
                self.totalPageSize = self.calculateTotalPageSize(totalCount);
                var startPage = ((parseInt(this.currentPageIndex.toString(), 10) - 5) < 1) ? 1 : (parseInt(this.currentPageIndex.toString(), 10) - 5);
                var endPage = ((parseInt(this.currentPageIndex.toString(), 10) + 5) > this.totalPageSize) ? this.totalPageSize : (parseInt(this.currentPageIndex.toString(), 10) + 5);
                self.setupPagingEvents(selector, filters);
                selector.querySelectorAll("li").each(function (idx, elem) {
                    if (elem.first("a").attribute("title") != "Previous" && elem.first("a").attribute("title") != "Next" && elem.first("a").attribute("title") != "First" && elem.first("a").attribute("title") != "Last") {
                        elem.drop();
                    }
                });
                for (let i = 1; i <= self.totalPageSize; i++) {
                    if (i < startPage || i > endPage) {
                        continue;
                    }
                    let li = document.create("<li />");
                    let idx = i;
                    let link = document.create(`<a class="button button-outline">${idx}</a>`).attribute("href", "#").attribute("title", idx).addEvent("click", function (e) {
                        e.preventDefault();
                        self.pageListing(this.attribute("title"), filters);
                    });
                    li.appendChild(link);
                    selector.insertBefore(li, selector.querySelector("li > a[title='Next']").parent());
                }
                if (self.totalPageSize > 0) {
                    selector.querySelector(`li > a[title='${self.currentPageIndex}']`).removeClass("button-outline").addClass("button-clear");
                }
                if (selector.querySelectorAll("li").length <= 5) {
                    selector.hide();
                }
                else {
                    selector.show();
                }
                if (self._elem.querySelector(`[data-m-segment='recordcount']`) != null) {
                    if (this.totalPageSize != null && this.totalPageSize > 0) {
                        let isOne = (this.totalPageSize === 1) ? "1" : this.currentPageIndex;
                        self._elem.querySelector(`[data-m-segment='recordcount']`).innerHTML = `<label class="record-count">${isOne} of ${this.totalPageSize} Pages (${totalCount} Total Records)</label>`;
                    }
                    else {
                        self._elem.querySelector(`[data-m-segment='recordcount']`).innerHTML = `<label class="record-count">No records found...</label>`;
                    }
                }
            }
            else {
                selector.hide();
            }
            self.applyPageSizeEvents(selector);
        }
        applyPageSizeEvents(selector) {
            var self = this;
            var parent = selector.parent();
            var control = parent.querySelector("[data-m-segment='controls'] [data-m-segment='paging']");
            control.val(self.pageSize);
            control.addEvent("change", function (e) {
                self.pageSize = control.val();
                schema_1.m.config["config.options.pageSize"] = self.pageSize;
                let storage = new storage_1.store(schema_1.DB, schema_1.DBVERSION, schema_1.STORE);
                storage.init().then((result) => {
                    return storage.setItem("metron.config", schema_1.m.config);
                }).then((result) => {
                    self.callListing();
                }).catch((reason) => {
                    console.log(`Error: Failed to access storage. ${reason}`);
                    self.callListing();
                });
            }, true);
        }
        calculateTotalPageSize(totalCount) {
            return Math.ceil(totalCount / this.pageSize);
        }
        pageListing(idx, filters) {
            this.currentPageIndex = idx;
            this.callListing();
        }
        getPreviousPage() {
            return (this.currentPageIndex === 0) ? this.currentPageIndex : (parseInt(this.currentPageIndex, 10) - 1);
        }
        getNextPage() {
            return (this.currentPageIndex === this.totalPageSize) ? this.currentPageIndex : (parseInt(this.currentPageIndex, 10) + 1);
        }
        setFilters() {
            var self = this;
            var elem = (self._elem != null) ? self._elem : document.querySelector(`[data-m-type='list'][data-m-model='${self.model}']`);
            var page = (elem != null) ? elem.attribute("data-m-page") : null;
            var routeName = routing_1.routing.getRouteName();
            var qs = web_1.web.querystring();
            if (qs != "") {
                self._filters = tools_1.tools.formatOptions(qs, schema_1.OptionTypes.QUERYSTRING);
            }
            let hash = routing_1.routing.getRouteUrl(self._filters);
            if (hash != null) {
                self.pageSize = (hash["PageSize"] != null) ? hash["PageSize"] : self.pageSize;
                self.currentPageIndex = (hash["PageIndex"] != null) ? hash["PageIndex"] : self.currentPageIndex;
                self.sortOrder = (hash["_SortOrder"] != null) ? hash["_SortOrder"] : self.sortOrder;
                self.sortDirection = (hash["_SortDirection"] != null) ? hash["_SortDirection"] : self.sortDirection;
                delete hash["PageSize"];
                delete hash["PageIndex"];
                delete hash["_SortOrder"];
                delete hash["_SortDirection"];
                if (routeName == null || routeName == page) {
                    for (let h in hash) {
                        if (hash.hasOwnProperty(h)) {
                            if (self._filters[h] == null) {
                                self._filters[h] = hash[h];
                            }
                        }
                    }
                }
            }
        }
        get elem() {
            return this._elem;
        }
        set elem(f) {
            this._elem = f;
        }
        get form() {
            return this._form;
        }
        set form(f) {
            this._form = f;
        }
        get filters() {
            return this._filters;
        }
        set filters(f) {
            this._filters = f;
        }
    }
    exports.list = list;
});
//# sourceMappingURL=lists.js.map