interface String {
    lower: () => string;
    upper: () => string;
    ltrim: () => string;
    rtrim: () => string;
    normalize: () => string;
    startsWith: (part: string) => boolean;
    endsWith: (part: string) => boolean;
    capFirst: () => string;
    capWords: () => string;
    truncateWords: (number: number) => string;
    truncateWordsWithHtml: (number: number) => string;
    stripHtml: () => string;
    escapeHtml: () => string;
    toBool: () => boolean;
    contains: (val: string) => boolean;
    slugify: (lower?: boolean) => string;
    toPhoneNumber: () => string;
    getValueByKey: (key: string) => string;
    setValueByKey: (key: string, replacement: string) => string;
}
interface StringConstructor {
    isNullOrEmpty: (val: any) => boolean;
}
interface Number {
    toBool: () => boolean;
    random: (min: number, max: number) => number;
}
interface Array<T> {
    empty: () => Array<any>;
    isEmpty: () => boolean;
    each: (callback: Function) => void;
    remove: (item: any) => any;
    contains: (partial: string, strict: boolean) => boolean;
    indexOfPartial: (partial: string) => number;
    toObjectArray: (objName: string) => Array<any>;
}
interface Object {
    extend: (dest: any, src: any) => any;
}
interface Document {
    selectOne: (selector: string) => Element;
    selectAll: (selector: string) => NodeListOf<Element>;
    create: (html: string) => Element;
}
interface NodeList {
    each: (callback: Function) => void;
    last: () => Element;
}
interface Element {
    attribute: (name: string, value?: string) => string & Element;
    parent: () => Element;
    first: (selector: string) => Element;
    append: (html: string) => Element;
    empty: () => Element;
    drop: () => Element;
    removeEvent: (event: string) => Element;
    addEvent: (event: string, callback: Function, overwrite?: boolean) => Element;
    show: (t?: string) => Element | void;
    hide: () => Element;
    toggle: () => Element;
    addClass: (className: string) => Element;
    removeClass: (className: string) => Element;
    asString: () => string;
    selectOne: (selector: string) => Element;
    selectAll: (selector: string) => NodeListOf<Element>;
    hasMatches: (selector: string) => boolean;
    up: (selector: string) => Element;
    isHidden: () => Boolean;
}
interface HTMLElement {
    clean: () => HTMLElement;
    val: (val?: string) => string;
}
interface XMLHttpRequest {
    responseJSON?: () => JSON;
}
declare type AjaxRequest = string | JSON | XMLDocument;
declare namespace metron {
    interface Dictionary {
        setItem: (key: string, value: any) => void;
        getItem: (key: string) => any;
        hasItem: (key: string) => boolean;
        removeItem: (key: string) => any;
        keys: () => Array<string>;
        values: () => Array<any>;
        each: (callback: Function) => void;
        clear: () => void;
    }
    interface Ajax {
        url: string;
        method: string;
        contentType: string;
        dataType: string;
        data: any;
        async: boolean;
        request: XMLHttpRequest;
        success?: (callback: Function) => XMLHttpRequest;
        error?: (callback: Function) => XMLHttpRequest;
        always?: (callback: Function) => XMLHttpRequest;
        send?: () => void;
    }
    interface Pivot {
        parent: HTMLElement;
        current: Element;
        next: Element;
        previous: Element;
    }
    interface ListOptions {
        mID?: string;
        hasRouting?: boolean;
    }
    interface FormOptions {
        mID?: string;
        hasRouting?: boolean;
    }
    enum OptionTypes {
        QUERYSTRING = 1,
        KEYVALUE = 2,
    }
    interface EventFunction {
        [name: string]: Function;
    }
    enum Event {
        INIT = 0,
        LOAD_FILTERS = 1,
        APPLY_VIEW_EVENTS = 2,
        POPULATE_LISTING = 3,
        UNDO_LAST = 4,
        CALL_LISTING = 5,
        SAVE = 6,
        LOAD_FORM = 7,
        LOAD_SELECTS = 8,
        CLEAR_FORM = 9,
        NEW = 10,
        UNDO = 11,
        DOWNLOAD = 12,
        EDIT = 13,
        DELETE = 14,
        CANCEL = 15,
    }
    const LIST = "list";
    const LOOKUP = "lookup";
    const FORM = "form";
    const VIEW = "view";
    const INFO = "info";
    const WARNING = "warning";
    const DANGER = "danger";
    const SUCCESS = "success";
    const DB = "metron.db";
    const DBVERSION = 1;
    const STORE = "metron.store";
}
declare var RSVP: any;
declare namespace metron {
    namespace dictionary {
    }
    namespace web {
        function querystring(obj?: any): Array<string> | string;
        function querystringify(obj: any, encode?: boolean): string;
        namespace cookie {
            function get(name: string): string;
            function set(name: string, val: string, date: Date): void;
            function remove(name: string): void;
        }
        namespace headers {
            function get(name: string): string;
        }
        function ajax(url: string, data?: any, method?: string, async?: boolean, contentType?: string, dataType?: string, success?: Function, failure?: Function, always?: Function): Ajax;
        function get(url: string, params?: any, contentType?: string, dataType?: string, success?: Function, failure?: Function, always?: Function): Ajax;
        function post(url: string, params?: any, contentType?: string, dataType?: string, success?: Function, failure?: Function, always?: Function): Ajax;
        function postAll(url: string, params?: any, contentType?: string, dataType?: string, success?: Function, failure?: Function, always?: Function): Ajax;
        function put(url: string, params?: any, contentType?: string, dataType?: string, success?: Function, failure?: Function, always?: Function): Ajax;
        function putAll(url: string, params?: any, contentType?: string, dataType?: string, success?: Function, failure?: Function, always?: Function): Ajax;
        function remove(url: string, params?: any, contentType?: string, dataType?: string, success?: Function, failure?: Function, always?: Function): Ajax;
    }
    namespace observer {
    }
    namespace guid {
        function newGuid(): string;
    }
}
declare var Awesomplete: any;
declare namespace metron {
    namespace controls {
        function polyfill(): void;
        function getPivot(name: string, callback?: Function): metron.controls.pivot;
        class pivots {
            static bindAll(callback?: Function): void;
        }
        class pivot {
            private pivotCollection;
            private displayIndex;
            private nextButton;
            private previousButton;
            private _name;
            private _pivotContainer;
            private _items;
            private _item;
            private _previousButton;
            private _nextButton;
            private _preEventFunctions;
            private _postEventFunctions;
            constructor(pivotCollection: Element, displayIndex?: number, nextButton?: any, previousButton?: any);
            private init(el);
            private applyPreEvent(el);
            addPreEvent(name: string, func: Function): void;
            addPostEvent(name: string, func: Function): void;
            removePostEvent(name: string): void;
            removePreEvent(name: string): void;
            next(): void;
            previous(): void;
            exact(target: number | string): boolean;
        }
    }
}
declare namespace metron {
    class store {
        localDBName: string;
        localDBVersion: number;
        localDBStore: string;
        private _db;
        private _hasIndexedDB;
        constructor(localDBName?: string, localDBVersion?: number, localDBStore?: string);
        init(): any;
        private getObjectStore();
        getItem(s: string, val?: string): any;
        setItem(s: string, a: any): any;
        removeItem(s: string): any;
        clearItems(): boolean;
    }
}
declare namespace metron {
    namespace tools {
        function reduceObject<T>(key: string, obj: T[]): Array<any>;
        function normalizeModelItems(obj: any, model: string, first?: boolean): Array<any>;
        function normalizeModelData(obj: any): any;
        function formatOptions(attr: string, opt?: OptionTypes): any;
        function cleanURL(url: string): string;
        function normalizeURL(url: string): string;
        function loadJSON(url: string, callback: Function): void;
        function getMatching(text: string, regex: RegExp): string;
        function getDataPrimary(key: string, values: string): any;
        function formatMessage(message: string, length?: number): string;
        function eventEnumToString(e: metron.Event): string;
        function formatDecimal(num: number): string;
        function formatDate(datetime: string): string;
        function formatDateTime(datetime: string): string;
        function formatTime(datetime: string | Date, isFullDate?: boolean): string;
        function formatBoolean(b: string): string;
        function convertDateStringToDate(datetime: string): Date;
    }
}
declare namespace metron {
    abstract class base {
        model: string;
        private baseType;
        protected _name: string;
        id: string;
        pivot: metron.controls.pivot;
        constructor(model: string, baseType: string);
        protected attachPivot(elem: Element): metron.controls.pivot;
        inject(type: string, method: string, func: Function): base;
        on(method: metron.Event, func: Function, overwrite?: boolean): base;
        action(action: string, model: string, func: Function): base;
        clearAlerts(): void;
        showAlerts(className: string, txt: string, jsn?: any, xml?: XMLDocument): void;
        shouldRoute(options: any): boolean;
    }
    class component extends base {
        static action(action: string, prefix: string, func: Function): void;
    }
}
declare namespace metron {
    namespace routing {
        function setRouteUrl(name: string, wsqs: string, wantsReplaceHash?: boolean): void;
        function getRouteUrl(filters?: any): any;
        function getRouteName(): string;
    }
}
declare namespace metron {
    var config: any;
    var globals: any;
    function onready(callback: Function, appName?: string): void;
    function load(segment: string, model: string, func: Function, name?: string): void;
    function ifQuerystring(callback: Function): void;
    namespace fw {
        function getApplicationRoot(page: string): string;
        function getApplicationName(page: string): string;
        function getBaseUrl(): string;
        function getAppUrl(): string;
        function getBaseAPI(): string;
        function getAPIExtension(): string;
        function getAPIURL(model: string): string;
        function loadOptionalFunctionality(): void;
    }
}
declare namespace metron {
    class lists {
        static bindAll(callback: Function): void;
    }
    class list<T> extends base {
        model: string;
        options: metron.ListOptions;
        private _elem;
        private _filters;
        private _items;
        private _rowTemplate;
        private _form;
        id: string;
        gTypeName: string;
        recycleBin: Array<T>;
        currentPageIndex: number;
        pageSize: number;
        totalPageSize: number;
        totalCount: number;
        sortOrder: string;
        sortDirection: string;
        private _defaults;
        fetchURL: string;
        constructor(model: string, options?: metron.ListOptions);
        init(): list<T>;
        private loadFilters(filters);
        private applyViewEvents();
        private withDefaults(parameters);
        populateListing(): void;
        undoLast(): void;
        formatData(item: T, isTable?: boolean): string;
        callListing(): void;
        populateTable(selector: Element): void;
        clearTable(selector: Element): void;
        getRows(selector: string): number;
        setupPagingEvents(selector: Element, filters?: any): void;
        createPaging(selector: Element, totalCount: any, filters?: any): void;
        private applyPageSizeEvents(selector);
        private calculateTotalPageSize(totalCount);
        private pageListing(idx, filters?);
        private getPreviousPage();
        private getNextPage();
        private setFilters();
        elem: Element;
        form: string;
        filters: T;
    }
}
declare namespace metron {
    class forms {
        static bindAll(callback: Function): void;
    }
    class form<T> extends base {
        model: string;
        options: metron.FormOptions;
        private _elem;
        private _fields;
        private _defaults;
        id: string;
        gTypeName: string;
        hasLoaded: boolean;
        constructor(model: string, options?: metron.FormOptions);
        private loadDefaults();
        init(toggle?: boolean): form<T>;
        save(data: T, pivotPosition: number, saveElement: Element): void;
        loadForm(parameters?: any, defaults?: any): void;
        loadSelects(selects: NodeListOf<Element>, callback?: Function): void;
        clearForm(selector?: string, callback?: Function): void;
        isValid(selector?: string): boolean;
        elem: Element;
    }
}
declare namespace metron {
    class views {
        static bindAll(): void;
    }
    class view<T> extends base {
        model: string;
        private _elem;
        private _form;
        id: string;
        gTypeName: string;
        constructor(model: string);
        init(): view<T>;
        callView(querystring: string): void;
        elem: Element;
        form: string;
    }
}
declare namespace metron {
    namespace templates {
        function load(includesFile: string): any;
        namespace list {
            function row<T>(template: string, item: T, isTable?: boolean): string;
            function format(type: string, val: string | Element, options?: any): string;
        }
        namespace markdown {
            function toHTML(src: string): string;
        }
        namespace master {
            function hasMaster(page: string): boolean;
            function loadMaster(page: string): any;
            function merge(template: string): void;
            function getContentRoot(): string;
        }
        function register(n: string, func: Function): void;
    }
}
