(function (factory) {
    if (typeof module === "object" && typeof module.exports === "object") {
        var v = factory(require, exports);
        if (v !== undefined) module.exports = v;
    }
    else if (typeof define === "function" && define.amd) {
        define(["require", "exports", "./tools", "./web", "./schema", "./routing"], factory);
    }
})(function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    const tools_1 = require("./tools");
    const web_1 = require("./web");
    const schema_1 = require("./schema");
    const routing_1 = require("./routing");
    class page {
        static action(action, prefix, func) {
            var actionName = (prefix != null) ? `${prefix}_${action}` : action;
            schema_1.m.globals.actions[actionName.lower()] = func;
        }
        static loadSelects(selects, callback, reload = false) {
            function populateSelects(el, rl) {
                let rli = rl;
                let node = el;
                let binding = el.attribute("data-m-binding");
                let key = (el.attribute("data-m-key")) != null ? el.attribute("data-m-key") : el.attribute("name");
                let nm = el.attribute("name");
                let nText = el.attribute("data-m-text");
                let options = (el.attribute("data-m-options") != null) ? tools_1.tools.formatOptions(el.attribute("data-m-options")) : {};
                return new Promise((resolve, reject) => {
                    if (rli || (binding != null && node.querySelectorAll("option").length <= 1)) {
                        web_1.web.get(`${routing_1.routing.getAPIURL(binding)}${web_1.web.querystringify(options)}`, {}, null, "json", function (data) {
                            data.each(function (i, item) {
                                node.append(`<option value="${item[key]}">${item[nText]}</option>`);
                            });
                            resolve(data);
                        });
                    }
                    else {
                        resolve();
                    }
                });
            }
            var promises = [];
            selects.each(function (indx, el) {
                promises.push(populateSelects(el, reload));
            });
            Promise.all(promises).then(() => {
                if (callback != null) {
                    callback();
                }
                if (self.loadSelects_m_inject != null) {
                    self.loadSelects_m_inject();
                }
            }).catch((reason) => {
                console.log("Error: Promise execution failed!");
            });
        }
        static loadActions(actions) {
            actions.each(function (indx, el) {
                let a = el.attribute("data-m-action");
                if (el.up("[data-m-segment='controls']") == null && el.up("[data-m-type='row']") == null) {
                    let ev;
                    if (el.nodeName.lower() === "a" || el.nodeName.lower() === "button" || el.nodeName.lower() === "div" || el.nodeName.lower() === "span" || (el.nodeName.lower() === "input" && (el.attribute("type").lower() === "submit" || el.attribute("type").lower() === "button"))) {
                        ev = "click";
                    }
                    else {
                        ev = "change";
                    }
                    el.addEvent(ev, function (e) {
                        e.preventDefault();
                        schema_1.m.globals.actions[a.lower()]();
                    }, true);
                }
            });
        }
        static bindActions() {
            page.loadActions(document.querySelectorAll("[data-m-action]"));
        }
        static clearAlerts(selector) {
            var elem = document.querySelector(selector);
            elem.innerHTML = "";
            elem.removeClass("info").removeClass("warning").removeClass("danger").removeClass("success");
            elem.attribute("data-m-state", "hide");
            elem.hide();
        }
        static showAlerts(selector, className, txt, jsn, xml) {
            var elem = document.querySelector(selector);
            elem.innerHTML = txt;
            elem.addClass(className);
            elem.attribute("data-m-state", "show");
            elem.show();
        }
        static clearFilters(selector) {
            selector.querySelectorAll("[data-m-action='filter'],.custom-filter").each((idx, elem) => {
                try {
                    elem.val("");
                }
                catch (e) {
                    console.log(`Could not clear value for ${(elem != null) ? elem.outerHTML : idx}: ${e}`);
                }
            });
        }
        get elem() {
            if (this._elem == null) {
                try {
                    this._elem = document.querySelector("[data-m-root]");
                }
                catch (e) {
                    console.log(e);
                }
            }
            return this._elem;
        }
    }
    exports.page = page;
});
//# sourceMappingURL=page.js.map