(function (factory) {
    if (typeof module === "object" && typeof module.exports === "object") {
        var v = factory(require, exports);
        if (v !== undefined) module.exports = v;
    }
    else if (typeof define === "function" && define.amd) {
        define(["require", "exports", "./tools", "./schema", "./web"], factory);
    }
})(function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    const tools_1 = require("./tools");
    const schema_1 = require("./schema");
    const web_1 = require("./web");
    var routing;
    (function (routing) {
        function config(options) {
            schema_1.m.globals.pager.root = options && options.root ? '/' + clearSlashes(options.root) + '/' : '/';
        }
        routing.config = config;
        function getFragment() {
            let match = window.location.href.match(/#(.*)$/);
            let fragment = match ? match[1] : "";
            if (fragment.lastIndexOf("/") != -1) {
                fragment = fragment.substring(0, fragment.lastIndexOf("/"));
            }
            return clearSlashes(fragment);
        }
        routing.getFragment = getFragment;
        function clearSlashes(path) {
            return path.toString().replace(/\/$/, "").replace(/^\//, "");
        }
        routing.clearSlashes = clearSlashes;
        function add(re, handler) {
            if (typeof re == "function") {
                handler = re;
                re = "";
            }
            schema_1.m.globals.pager.pages.push({ re: re, handler: handler });
        }
        routing.add = add;
        function remove(param) {
            for (let i = 0, r; i < schema_1.m.globals.pager.pages.length, r = schema_1.m.globals.pager.pages[i]; i++) {
                if (r.handler === param || r.re.toString() === param.toString()) {
                    schema_1.m.globals.pager.pages.splice(i, 1);
                }
            }
        }
        routing.remove = remove;
        function flush() {
            schema_1.m.globals.pager.pages = [];
            schema_1.m.globals.pager.root = "/";
        }
        routing.flush = flush;
        function check(f) {
            var fragment = f || getFragment();
            for (var i = 0; i < schema_1.m.globals.pager.pages.length; i++) {
                var match = fragment.match(schema_1.m.globals.pager.pages[i].re);
                if (match) {
                    match.shift();
                    schema_1.m.globals.pager.pages[i].handler.apply({}, match);
                    return schema_1.m.globals.pager;
                }
            }
            return schema_1.m.globals.pager;
        }
        routing.check = check;
        function listen() {
            var current = getFragment();
            var fn = function () {
                if (current !== getFragment()) {
                    current = getFragment();
                    check(current);
                }
            };
            clearInterval(schema_1.m.globals.pager.interval);
            schema_1.m.globals.pager.interval = setInterval(fn, 50);
        }
        routing.listen = listen;
        function navigate(path, replace = false, callback) {
            path = path ? path : '';
            if (replace) {
                history.replaceState(null, null, `#/${path}/`);
            }
            else {
                history.pushState(null, null, `#/${path}/`);
            }
            if (callback !== undefined) {
                callback();
            }
        }
        routing.navigate = navigate;
        function setRouteUrl(name, wsqs, wantsReplaceHash = false) {
            var hash = (wsqs.length > 1) ? wsqs.substr(1) : "";
            if (hash != "" && document.location.search != null) {
                try {
                    let hashItems = tools_1.tools.formatOptions(hash, schema_1.OptionTypes.QUERYSTRING);
                    let qsItems = tools_1.tools.formatOptions((document.location.search.startsWith("?") ? document.location.search.substr(1) : document.location.search), schema_1.OptionTypes.QUERYSTRING);
                    for (let h in hashItems) {
                        if (hashItems.hasOwnProperty(h)) {
                            if (qsItems[h] != null) {
                                delete hashItems[h];
                            }
                        }
                    }
                    hash = web_1.web.querystringify(hashItems).substr(1);
                }
                catch (e) {
                    console.log(`Error: failed to parse query string and hash. ${e}`);
                }
            }
            if (name != null) {
                hash = `/${name}/${hash}`;
            }
            schema_1.m.globals.hashLoadedFromApplication = true;
            if (hash != null && hash != "") {
                (wantsReplaceHash) ? document.location.hash = `#${hash}` : history.replaceState({}, "", `#${hash}`);
            }
        }
        routing.setRouteUrl = setRouteUrl;
        function getRouteUrl(filters) {
            var result = web_1.web.hash();
            if (filters != null) {
                for (let h in result) {
                    if (result.hasOwnProperty(h)) {
                        if (filters[h] != null) {
                            delete result[h];
                        }
                    }
                }
            }
            return result;
        }
        routing.getRouteUrl = getRouteUrl;
        function getRouteName() {
            var hash = document.location.hash;
            if (hash.substr(0, 1) == "#") {
                hash = hash.substr(1);
            }
            if (hash.substr(0, 1) == "/") {
                hash = hash.substr(1);
            }
            if (hash.length > 1) {
                if (hash.indexOf("/") != -1) {
                    try {
                        hash = hash.split("/")[0];
                    }
                    catch (e) {
                        console.log(`Error: Failed to get routing. ${e}`);
                        return null;
                    }
                }
                return hash;
            }
            return null;
        }
        routing.getRouteName = getRouteName;
        function getApplicationRoot(page) {
            let root = (document.querySelector("body[data-m-root]") != null) ? `${document.querySelector("body[data-m-root]").attribute("data-m-root")}` : null;
            if (root == null) {
                root = tools_1.tools.getMatching(page, /\{\{m:root=\"(.*)\"\}\}/g);
            }
            schema_1.m.config["config.root"] = (root != null) ? root : "";
            return root;
        }
        routing.getApplicationRoot = getApplicationRoot;
        function getApplicationName(page) {
            let appName = (document.querySelector("body[data-m-page]") != null) ? `${document.querySelector("body[data-m-page]").attribute("data-m-page")}` : null;
            if (appName == null) {
                appName = tools_1.tools.getMatching(page, /\{\{m:page=\"(.*)\"\}\}/g);
            }
            schema_1.m.config["config.appName"] = (appName != null) ? appName : "";
            return appName;
        }
        routing.getApplicationName = getApplicationName;
        function getBaseUrl() {
            if (schema_1.m.config["config.baseURL"] != null) {
                return (schema_1.m.config["config.baseURL"].endsWith("/")) ? schema_1.m.config["config.baseURL"].substr(0, schema_1.m.config["config.baseURL"].length - 2) : `${schema_1.m.config["config.baseURL"]}`;
            }
            return "";
        }
        routing.getBaseUrl = getBaseUrl;
        function getAppUrl() {
            if (schema_1.m.config["config.baseURL"] != null) {
                let url = (schema_1.m.config["config.baseURL"].endsWith("/")) ? schema_1.m.config["config.baseURL"].substr(0, schema_1.m.config["config.baseURL"].length - 2) : `${schema_1.m.config["config.baseURL"]}`;
                return (schema_1.m.config["config.root"] != null && schema_1.m.config["config.root"] != "") ? `${url}/${schema_1.m.config["config.root"]}` : url;
            }
            return "";
        }
        routing.getAppUrl = getAppUrl;
        function getBaseAPI() {
            if (schema_1.m.config["config.api.dir"] != null) {
                let url = (schema_1.m.config["config.api.dir"].endsWith("/")) ? schema_1.m.config["config.api.dir"].substr(0, schema_1.m.config["config.api.dir"].length - 2) : `${schema_1.m.config["config.api.dir"]}`;
                return (schema_1.m.config["config.root"] != null && schema_1.m.config["config.root"] != "") ? `${schema_1.m.config["config.root"]}/${url}` : url;
            }
            return "";
        }
        routing.getBaseAPI = getBaseAPI;
        function getAPIExtension() {
            if (schema_1.m.config["config.api.extension"] != null) {
                return schema_1.m.config["config.api.extension"];
            }
            return "";
        }
        routing.getAPIExtension = getAPIExtension;
        function getAPIURL(model) {
            return `${getBaseUrl()}/${getBaseAPI()}/${model}${getAPIExtension()}`;
        }
        routing.getAPIURL = getAPIURL;
    })(routing = exports.routing || (exports.routing = {}));
});
//# sourceMappingURL=routing.js.map