(function (factory) {
    if (typeof module === "object" && typeof module.exports === "object") {
        var v = factory(require, exports);
        if (v !== undefined) module.exports = v;
    }
    else if (typeof define === "function" && define.amd) {
        define(["require", "exports", "./schema"], factory);
    }
})(function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    const schema_1 = require("./schema");
    class store {
        constructor(localDBName, localDBVersion, localDBStore) {
            this.localDBName = localDBName;
            this.localDBVersion = localDBVersion;
            this.localDBStore = localDBStore;
            this._hasIndexedDB = false;
            var self = this;
            if (localDBName == null) {
                self.localDBName = schema_1.m.config["config.storage.localDBName"];
            }
            if (localDBVersion == null) {
                self.localDBVersion = schema_1.m.config["config.storage.localDBVersion"];
            }
            if (localDBStore == null) {
                self.localDBStore = schema_1.m.config["config.storage.localDBStore"];
            }
        }
        init() {
            var self = this;
            var p = new Promise(function (resolve, reject) {
                try {
                    if (window.indexedDB == null) {
                        window.indexedDB = window.indexedDB || window.mozIndexedDB || window.webkitIndexedDB || window.msIndexedDB;
                        window.IDBTransaction = window.IDBTransaction || window.webkitIDBTransaction || window.msIDBTransaction || { READ_WRITE: "readwrite" };
                        window.IDBKeyRange = window.IDBKeyRange || window.webkitIDBKeyRange || window.msIDBKeyRange;
                    }
                    if (!window.indexedDB) {
                        console.log("Warning: IndexedDB is unavailable. Using localStorage instead.");
                    }
                    else {
                        self._hasIndexedDB = true;
                        let request = window.indexedDB.open(self.localDBName, self.localDBVersion);
                        request.onerror = function (evt) {
                            self._hasIndexedDB = false;
                            console.log("Warning: Access to IndexedDB for application has been rejected.");
                        };
                        request.onupgradeneeded = function (evt) {
                            let objectStore = evt.currentTarget.result.createObjectStore(self.localDBStore, { keyPath: "name" });
                            objectStore.createIndex("name", "name", { unique: true });
                            objectStore.transaction.oncomplete = function (oevt) {
                                console.log(`Info: Object store has been successfully created. ${objectStore}`);
                            };
                        };
                        request.onsuccess = function (evt) {
                            self._db = evt.target.result;
                            console.log(`Info: Database initialized. ${self._db}`);
                            resolve(self._db);
                        };
                    }
                }
                catch (e) {
                    console.log(`Error: Failed to initialize storage. ${e}`);
                    reject(e);
                }
            });
            return p;
        }
        getObjectStore() {
            var self = this;
            try {
                let transaction = self._db.transaction(self.localDBStore, "readwrite");
                return transaction.objectStore(self.localDBStore);
            }
            catch (e) {
                console.log(`Error: Failed to get object store. ${e}`);
                return null;
            }
        }
        getItem(s, val) {
            var self = this;
            var p = new Promise(function (resolve, reject) {
                try {
                    if (window.indexedDB != null) {
                        let objectStore = self.getObjectStore();
                        var request = objectStore.get(s);
                        request.onsuccess = function (evt) {
                            if (val != null && evt.target.result != null) {
                                resolve(evt.target.result[val]);
                            }
                            else {
                                resolve(evt.target.result);
                            }
                        };
                    }
                    else if (sessionStorage != null) {
                        resolve(sessionStorage.getItem(s));
                    }
                }
                catch (e) {
                    console.log(`Error: Failed to retrieve item. ${e}`);
                    reject(e);
                }
            });
            return p;
        }
        setItem(s, a) {
            var self = this;
            var p = new Promise(function (resolve, reject) {
                try {
                    if (window.indexedDB != null) {
                        let objectStore = self.getObjectStore();
                        if (!(typeof a === "string")) {
                            let b = {};
                            for (let prop in a) {
                                if (a.hasOwnProperty(prop) && (!(a[prop] instanceof Function))) {
                                    b[prop] = a[prop];
                                }
                            }
                            delete b["__proto__"];
                            a = JSON.stringify(b);
                        }
                        let item = { "name": s, "value": a };
                        var request = objectStore.put(item);
                        request.onsuccess = function (evt) {
                            console.log(`Info: Item added to the object store. ${evt.target.result}`);
                            resolve(evt.target.result);
                        };
                    }
                    else if (sessionStorage != null) {
                        sessionStorage.setItem(s, a);
                        resolve(true);
                    }
                }
                catch (e) {
                    console.log(`Error: Failed to set item. ${e}`);
                    reject(e);
                }
            });
            return p;
        }
        removeItem(s) {
            var self = this;
            var p = new Promise(function (resolve, reject) {
                try {
                    if (window.indexedDB != null) {
                        var request = self.getObjectStore().delete(s);
                        request.onsuccess = function (evt) {
                            console.log("Info: Object deleted.");
                            resolve(true);
                        };
                    }
                    else if (sessionStorage != null) {
                        sessionStorage.removeItem(s);
                        resolve(true);
                    }
                }
                catch (e) {
                    console.log(`Error: Failed to remove item. ${e}`);
                    reject(e);
                }
            });
            return p;
        }
        clearItems() {
            var self = this;
            try {
                if (window.indexedDB != null) {
                    self.getObjectStore().clear();
                }
                if (sessionStorage != null) {
                    sessionStorage.clear();
                    return true;
                }
            }
            catch (e) {
                console.log(`Error: Failed to clear items. ${e}`);
                return false;
            }
            return false;
        }
    }
    exports.store = store;
});
//# sourceMappingURL=storage.js.map