(function (factory) {
    if (typeof module === "object" && typeof module.exports === "object") {
        var v = factory(require, exports);
        if (v !== undefined) module.exports = v;
    }
    else if (typeof define === "function" && define.amd) {
        define(["require", "exports", "./tools", "./web", "./schema", "./routing"], factory);
    }
})(function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    const tools_1 = require("./tools");
    const web_1 = require("./web");
    const schema_1 = require("./schema");
    const routing_1 = require("./routing");
    var templates;
    (function (templates) {
        function load(includesFile) {
            var url = `${routing_1.routing.getAppUrl()}/${includesFile}`;
            return new Promise((resolve, reject) => {
                web_1.web.ajax(url, {}, "GET", true, "text/html", "text", (content) => {
                    resolve(content);
                }, (txt, jsn, xml) => {
                    console.log(`Error loading ${includesFile}: ${txt}`);
                    reject(txt);
                });
            });
        }
        templates.load = load;
        function merge(template, item, isTable = true) {
            var result = template;
            for (let k in item) {
                if (item.hasOwnProperty(k)) {
                    let replacement = `{{${k}}}`;
                    result = result.replace(new RegExp(replacement, "g"), (item[k] != null && item[k] != "null") ? item[k] : "");
                }
            }
            var doc = document.createElement((isTable ? "tbody" : "div"));
            doc.innerHTML = result;
            doc.querySelectorAll("[data-m-format]").each((idx, elem) => {
                let options = (elem.attribute("data-m-options") != null) ? tools_1.tools.formatOptions(elem.attribute("data-m-options")) : null;
                if (elem.firstElementChild == null) {
                    elem.innerText = format(elem.attribute("data-m-format"), elem.innerText, options);
                }
                else {
                    format(elem.attribute("data-m-format"), elem, options);
                }
            });
            return doc.innerHTML;
        }
        templates.merge = merge;
        function format(type, val, options) {
            switch (type.lower()) {
                case "yesno":
                    return tools_1.tools.formatBoolean(val);
                case "datetime":
                    return tools_1.tools.formatDateTime(val);
                case "date":
                    return tools_1.tools.formatDate(val);
                case "time":
                    return tools_1.tools.formatTime(val);
                case "formatmessage":
                    return tools_1.tools.formatMessage(val, options["length"], options["closetags"]);
                case "decimal":
                    return tools_1.tools.formatDecimal(val);
                default:
                    return schema_1.m.globals[type](val, options);
            }
        }
        templates.format = format;
        let markdown;
        (function (markdown) {
            function toHTML(src) {
                let html = "";
                function escape(text) {
                    return new Option(text).innerHTML;
                }
                function inlineEscape(str) {
                    return str.replace(/!\[([^\]]*)]\(([^(]+)\)/g, '<img alt="$1" src="$2" />')
                        .replace(/\[([^\]]+)]\(([^(]+)\)/g, '$1'.link('$2'))
                        .replace(/`([^`]+)`/g, (match, p1, offset, string) => {
                        return `<code>${escape(p1)}</code>`;
                    })
                        .replace(/\*\*([^*]+)\*\*/g, '<strong>$1</strong>')
                        .replace(/\*([^*]+)\*/g, '<em>$1</em>')
                        .replace(/  \n/g, '<br />');
                }
                function processWrappedMarkdown(prependType, line) {
                    return prependType[1] + ("\n" + line)
                        .split(prependType[0])
                        .slice(1)
                        .map(prependType[3] ? escape : inlineEscape)
                        .join(prependType[3] || "</li>\n<li>") + prependType[2];
                }
                function processSemanticMarkdown(char, line) {
                    return (char == "#")
                        ? ("<h" + (char = line.indexOf(" ")) + ">" + inlineEscape(line.slice(char + 1)) + "</h" + char + ">")
                        : (char == "<" ? line : "<p>" + inlineEscape(line) + "</p>");
                }
                src.replace(/&gt;/g, ">").replace(/^\s+|\r|\s+$/g, "").replace(/\t/g, "    ").split(/\n\n+/).forEach((line, idx, lines) => {
                    let char = line[0];
                    let prependType = {
                        '*': [/\n\* /, "<ul><li>", "</li></ul>"],
                        '1': [/\n[1-9]\d*\.? /, "<ol><li>", "</li></ol>"],
                        ' ': [/\n    /, "<pre><code>", "</pre></code>", "\n"],
                        '>': [/\n> /, "<blockquote>", "</blockquote>", "\n"],
                    }[char];
                    html += prependType ? processWrappedMarkdown(prependType, line) : processSemanticMarkdown(char, line);
                });
                return html;
            }
            markdown.toHTML = toHTML;
        })(markdown = templates.markdown || (templates.markdown = {}));
        let master;
        (function (master) {
            function hasMaster(page) {
                if (page.match(/\{\{m:master=\"(.*)\"\}\}/g) != null && page.match(/\{\{m:master=\"(.*)\"\}\}/g).length > 0) {
                    return true;
                }
                return false;
            }
            master.hasMaster = hasMaster;
            function loadMaster(page) {
                return new Promise(function (resolve, reject) {
                    if (master.hasMaster(page)) {
                        let root = tools_1.tools.getMatching(page, /\{\{m:root=\"(.*)\"\}\}/g);
                        let fileName = tools_1.tools.getMatching(page, /\{\{m:master=\"(.*)\"\}\}/g);
                        web_1.web.get(`${root}/${fileName}`, {}, "text/html", "text", (resp) => {
                            templates.master.merge(resp);
                            resolve(resp);
                        }, (err) => {
                            document.documentElement.append(`<h1>Error: Failed to load [${root}/${fileName}].</h1><p>${err}</p>`);
                            reject(err);
                        });
                    }
                    else {
                        resolve();
                    }
                });
            }
            master.loadMaster = loadMaster;
            function merge(template) {
                function _copyAttributes(src, elemName) {
                    for (let i = 0; i < src.attributes.length; i++) {
                        document.documentElement.querySelector(elemName).attribute(src.attributes[i].name, src.attributes[i].value);
                    }
                }
                let placeholder = document.createElement("html");
                let content = getContentRoot();
                placeholder.innerHTML = `<metron>${template.replace("{{m:content}}", content).replace(/head/g, "mhead").replace(/body/g, "mbody").replace(/mheader/g, "header")}</metron>`;
                document.documentElement.empty();
                if (document.documentElement.hasChildNodes()) {
                    document.querySelector("head").innerHTML = placeholder.querySelector("mhead").innerHTML;
                    _copyAttributes(placeholder.querySelector("mhead"), "head");
                    document.querySelector("body").innerHTML = placeholder.querySelector("mbody").innerHTML;
                    _copyAttributes(placeholder.querySelector("mbody"), "body");
                }
                else {
                    document.documentElement.append(placeholder.innerHTML);
                }
            }
            master.merge = merge;
            function getContentRoot() {
                if (document.documentElement.querySelector("body") != null) {
                    return document.documentElement.querySelector("body").innerHTML.replace(/\{\{m:root=\"(.*)\"\}\}/g, "").replace(/\{\{m:master=\"(.*)\"\}\}/g, "");
                }
                return document.documentElement.innerHTML.replace(/\{\{m:root=\"(.*)\"\}\}/g, "").replace(/\{\{m:master=\"(.*)\"\}\}/g, "");
            }
            master.getContentRoot = getContentRoot;
        })(master = templates.master || (templates.master = {}));
        function register(n, func) {
            schema_1.m.globals[n] = func;
        }
        templates.register = register;
    })(templates = exports.templates || (exports.templates = {}));
});
//# sourceMappingURL=templates.js.map