(function (factory) {
    if (typeof module === "object" && typeof module.exports === "object") {
        var v = factory(require, exports);
        if (v !== undefined) module.exports = v;
    }
    else if (typeof define === "function" && define.amd) {
        define(["require", "exports", "./schema"], factory);
    }
})(function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    const schema_1 = require("./schema");
    var tools;
    (function (tools) {
        function reduceObject(key, obj) {
            let resp = [];
            obj.forEach(function (val, idx) {
                for (let k in val) {
                    if (k === key) {
                        resp.push(val[k]);
                    }
                }
            });
            return resp;
        }
        tools.reduceObject = reduceObject;
        function normalizeModelItems(obj, model, first = false) {
            var result = obj;
            if (!first) {
                if (obj[model] != null) {
                    result = obj[model];
                }
            }
            else {
                if (obj[model] != null) {
                    result = obj[model][0];
                }
            }
            return result;
        }
        tools.normalizeModelItems = normalizeModelItems;
        function normalizeModelData(obj) {
            for (let k in obj) {
                if (obj[k] == null || obj[k] === "null") {
                    delete obj[k];
                }
            }
            return obj;
        }
        tools.normalizeModelData = normalizeModelData;
        function formatOptions(attr, opt = schema_1.OptionTypes.KEYVALUE) {
            var pairDivider;
            var optDivider;
            switch (opt) {
                case schema_1.OptionTypes.QUERYSTRING:
                    pairDivider = "&";
                    optDivider = "=";
                    break;
                default:
                    pairDivider = ";";
                    optDivider = ":";
                    break;
            }
            var pairs = attr.split(pairDivider);
            if (pairs[pairs.length - 1].trim() == "") {
                pairs.pop();
            }
            var result = "";
            for (let i = 0; i < pairs.length; i++) {
                let p = pairs[i].split(optDivider);
                try {
                    result += `"${p[0].trim()}":"${p[1].trim()}"`;
                    if (i != (pairs.length - 1)) {
                        result += ",";
                    }
                }
                catch (e) {
                    throw new Error("Error: Invalid key/value pair!");
                }
            }
            var response = null;
            try {
                response = JSON.parse(`{${result}}`);
            }
            catch (e) {
                throw new Error("Error: Invalid JSON for options!");
            }
            return response;
        }
        tools.formatOptions = formatOptions;
        function getMatching(text, regex) {
            let match = regex.exec(text);
            if (match[1] !== undefined) {
                if (match[1].contains("\"")) {
                    return match[1].split("\"")[0];
                }
                return match[1];
            }
            return null;
        }
        tools.getMatching = getMatching;
        function getDataPrimary(key, values) {
            return values.getValueByKey(key);
        }
        tools.getDataPrimary = getDataPrimary;
        function formatMessage(message, length, closetags) {
            try {
                let len = (length != null && length > 0) ? length : 15;
                if (closetags && typeof (message) !== "string") {
                    let result = `${message.innerHTML.truncateWordsWithHtml(len)}...`;
                    message.innerHTML = result;
                    return result;
                }
                if (message.split(" ").length > len) {
                    return `${message.truncateWords(len)}...`;
                }
            }
            catch (e) {
            }
            return message;
        }
        tools.formatMessage = formatMessage;
        function eventEnumToString(e) {
            switch (e) {
                case schema_1.Event.INIT:
                    return "init";
                case schema_1.Event.LOAD_FILTERS:
                    return "loadFilters";
                case schema_1.Event.APPLY_VIEW_EVENTS:
                    return "applyViewEvents";
                case schema_1.Event.POPULATE_LISTING:
                    return "populateListing";
                case schema_1.Event.UNDO_LAST:
                    return "undoLast";
                case schema_1.Event.CALL_LISTING:
                    return "callListing";
                case schema_1.Event.SAVE:
                    return "save";
                case schema_1.Event.LOAD_FORM:
                    return "loadForm";
                case schema_1.Event.LOAD_SELECTS:
                    return "loadSelects";
                case schema_1.Event.CLEAR_FORM:
                    return "clearForm";
                case schema_1.Event.NEW:
                    return "new";
                case schema_1.Event.UNDO:
                    return "undo";
                case schema_1.Event.DOWNLOAD:
                    return "download";
                case schema_1.Event.EDIT:
                    return "edit";
                case schema_1.Event.DELETE:
                    return "delete";
                case schema_1.Event.CANCEL:
                    return "cancel";
                case schema_1.Event.SAVING:
                    return "saving";
                default:
                    throw new Error("Error: Invalid enum.");
            }
        }
        tools.eventEnumToString = eventEnumToString;
        function formatDecimal(num) {
            let dec = parseFloat(num.toString());
            return dec.toFixed(2);
        }
        tools.formatDecimal = formatDecimal;
        function formatDate(datetime) {
            if (!String.isNullOrEmpty(datetime)) {
                let d = convertDateStringToDate(datetime);
                let m = d.getMonth() + 1;
                let mm = m < 10 ? "0" + m : m;
                let dd = d.getDate();
                let ddd = dd < 10 ? "0" + dd : dd;
                let y = d.getFullYear();
                return `${mm}-${ddd}-${y}`;
            }
            return "";
        }
        tools.formatDate = formatDate;
        function formatDateTime(datetime) {
            if (!String.isNullOrEmpty(datetime)) {
                let d = convertDateStringToDate(datetime);
                let m = d.getMonth() + 1;
                let mm = m < 10 ? "0" + m : m;
                let dd = d.getDate();
                let ddd = dd < 10 ? "0" + dd : dd;
                let y = d.getFullYear();
                let time = formatTime(d, true);
                return `${mm}-${ddd}-${y} ${time}`;
            }
            return "";
        }
        tools.formatDateTime = formatDateTime;
        function formatTime(datetime, isFullDate = false) {
            var d;
            if (isFullDate) {
                if (datetime.length) {
                    d = convertDateStringToDate(datetime);
                }
                else {
                    d = datetime;
                }
            }
            else {
                let c = datetime.split(":");
                d = new Date(new Date().getFullYear(), new Date().getMonth(), new Date().getDate(), parseInt(c[0], 10), parseInt(c[1], 10), parseInt(c[2], 10));
            }
            var h = d.getHours();
            var m = d.getMinutes();
            var ampm = h >= 12 ? "pm" : "am";
            h = h % 12;
            h = h ? h : 12;
            var mm = m < 10 ? "0" + m : m;
            var result = `${h}:${mm} ${ampm}`;
            return result;
        }
        tools.formatTime = formatTime;
        function convertDateStringToDate(datetime) {
            if (datetime.indexOf("T") > 0 && datetime.indexOf("-") > 0 && datetime.toLowerCase().indexOf("gmt") == -1) {
                let dateString = datetime.substring(0, datetime.indexOf("T"));
                let dateArray = dateString.split("-");
                let timeString = datetime.substring(datetime.indexOf("T") + 1, datetime.length);
                let timeArray = timeString.split(":");
                return new Date(parseInt(dateArray[0]), parseInt(dateArray[1]) - 1, parseInt(dateArray[2]), parseInt(timeArray[0], 10), parseInt(timeArray[1], 10), parseInt(timeArray[2], 10), 0);
            }
            else if (datetime.indexOf(" ") > 0 && datetime.indexOf("/") > 0) {
                let dateString = datetime.substring(0, datetime.indexOf(" "));
                let dateArray = dateString.split("/");
                let timeString = datetime.substring(datetime.indexOf(" ") + 1, datetime.length);
                let timeArray = timeString.split(":");
                return new Date(parseInt(dateArray[2]), parseInt(dateArray[0]) - 1, parseInt(dateArray[1]), parseInt(timeArray[0], 10), parseInt(timeArray[1], 10), parseInt(timeArray[2], 10), 0);
            }
            return new Date(datetime);
        }
        tools.convertDateStringToDate = convertDateStringToDate;
        function formatBoolean(b) {
            if (b.toBool()) {
                return "yes";
            }
            return "no";
        }
        tools.formatBoolean = formatBoolean;
    })(tools = exports.tools || (exports.tools = {}));
});
//# sourceMappingURL=tools.js.map