(function (factory) {
    if (typeof module === "object" && typeof module.exports === "object") {
        var v = factory(require, exports);
        if (v !== undefined) module.exports = v;
    }
    else if (typeof define === "function" && define.amd) {
        define(["require", "exports", "./tools", "./schema"], factory);
    }
})(function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    const tools_1 = require("./tools");
    const schema_1 = require("./schema");
    var web;
    (function (web) {
        function parseUrl(url, obj, encode = false) {
            let paramPairs = [];
            if (url.contains('?')) {
                let parts = url.split('?');
                url = parts[0];
                paramPairs = paramPairs.concat(parts[1].split('&'));
            }
            for (let prop in obj) {
                if (obj.hasOwnProperty(prop) && !paramPairs.contains(prop, true)) {
                    let item = (encode) ? encodeURIComponent(obj[prop]) : obj[prop];
                    paramPairs.push(prop + '=' + item);
                }
                else if (obj.hasOwnProperty(prop) && paramPairs.contains(prop, true)) {
                    let item = (encode) ? encodeURIComponent(obj[prop]) : obj[prop];
                    paramPairs[paramPairs.indexOfPartial(prop)] = prop + '=' + item;
                }
            }
            return url + '?' + paramPairs.join('&');
        }
        function querystring(obj) {
            if (typeof (document) !== 'undefined') {
                if (typeof (obj) === 'string' && arguments.length === 1) {
                    let result = [];
                    let match;
                    let re = new RegExp('(?:\\?|&)' + obj + '=(.*?)(?=&|$)', 'gi');
                    while ((match = re.exec(document.location.search)) !== null) {
                        result.push(match[1]);
                    }
                    return result;
                }
                else if (typeof (obj) === 'string' && arguments.length > 1) {
                    return [parseUrl(obj, arguments[1])];
                }
                else if (obj != null) {
                    return [parseUrl(document.location.href, obj)];
                }
                else {
                    return document.location.search.substring(1);
                }
            }
            else {
                throw 'Error: No document object found. Environment may not contain a DOM.';
            }
        }
        web.querystring = querystring;
        function hash(key) {
            var hash = document.location.hash;
            if (hash.substr(0, 1) == "#") {
                hash = hash.substr(1);
            }
            if (hash.substr(0, 1) == "/") {
                hash = hash.substr(1);
            }
            if (hash.length > 1) {
                if (hash.indexOf("/") != -1) {
                    try {
                        hash = hash.split("/")[1];
                    }
                    catch (e) {
                        console.log(`Error: Failed to parse hash value: ${e}`);
                    }
                }
                try {
                    const result = tools_1.tools.formatOptions(hash, schema_1.OptionTypes.QUERYSTRING);
                    if (key != null) {
                        return [result[key]];
                    }
                    return result;
                }
                catch (e) {
                    console.log(`Error formatting has values: ${e}`);
                }
            }
        }
        web.hash = hash;
        function querystringify(obj, encode = false) {
            return parseUrl("", obj, encode);
        }
        web.querystringify = querystringify;
        function loadJSON(url, callback) {
            if (!url.contains("://")) {
                url = `${window.location.protocol}//${normalizeURL(window.location.host)}/${url}`;
            }
            web.get(`${url}`, {}, null, "JSON", function (data) {
                if (callback != null) {
                    callback(data);
                }
            });
        }
        web.loadJSON = loadJSON;
        function cleanURL(url) {
            if (url.startsWith("//")) {
                url = url.substring(1);
            }
            if (url.endsWith("?")) {
                url = url.substring(0, url.length - 1);
            }
            if (url == "/") {
                return "";
            }
            return url;
        }
        web.cleanURL = cleanURL;
        function normalizeURL(url) {
            if (url.endsWith("/")) {
                return url.substr(0, (url.length - 2));
            }
            return url;
        }
        web.normalizeURL = normalizeURL;
        let cookie;
        (function (cookie_1) {
            function get(name) {
                if (typeof (document) !== 'undefined') {
                    let cookieParts = document.cookie.split(';');
                    for (let i = 0; i < cookieParts.length; i++) {
                        let cookieName = cookieParts[i].substr(0, cookieParts[i].indexOf("="));
                        let cookieValue = cookieParts[i].substr(cookieParts[i].indexOf("=") + 1);
                        if (cookieName.trim() === name) {
                            return cookieValue;
                        }
                    }
                    return null;
                }
                else {
                    throw 'Error: No document object found. Environment may not contain a DOM.';
                }
            }
            cookie_1.get = get;
            function set(name, val, date) {
                if (typeof (document) !== 'undefined') {
                    let cookie = name + '=' + val + ';path="/"';
                    if (typeof (date) !== 'undefined') {
                        cookie += ';expires=' + date.toUTCString();
                    }
                    document.cookie = cookie;
                }
                else {
                    throw 'Error: No document object found. Environment may not contain a DOM.';
                }
            }
            cookie_1.set = set;
            function remove(name) {
                if (typeof (document) !== 'undefined') {
                    document.cookie = name + "=; expires=Thu, 01 Jan 1970 00:00:00 UTC";
                }
                else {
                    throw 'Error: No document object found. Environment may not contain a DOM.';
                }
            }
            cookie_1.remove = remove;
        })(cookie = web.cookie || (web.cookie = {}));
        let headers;
        (function (headers) {
            function get(name) {
                if (typeof (document) !== 'undefined') {
                    let request = new XMLHttpRequest();
                    request.open("HEAD", document.location.href, false);
                    request.send(null);
                    if (name !== undefined) {
                        return request.getResponseHeader(name);
                    }
                    else {
                        return request.getAllResponseHeaders();
                    }
                }
                else {
                    throw 'Error: No document object found. Environment may not contain a DOM.';
                }
            }
            headers.get = get;
        })(headers = web.headers || (web.headers = {}));
        function ajax(url, data = {}, method = "POST", async = true, contentType = "application/x-www-form-urlencoded; charset=UTF-8", dataType = "text", success, failure, always, headers) {
            function _parseResult(request) {
                switch (dataType.lower()) {
                    case "text":
                        return request.responseText;
                    case "json":
                        return request.responseJSON();
                    case "xml":
                        return request.responseXML;
                    default:
                        return request.responseText;
                }
            }
            function _send(request, data) {
                var ajx = this;
                request.open(method, url, async, data);
                request.onreadystatechange = function () {
                    if (request.readyState === 4) {
                        if (request.status === 200) {
                            if (success !== undefined) {
                                success(_parseResult(request));
                            }
                        }
                        if (request.status === 404 || request.status === 405 || request.status === 500) {
                            if (failure !== undefined) {
                                failure(request.responseText, request.responseJSON(), request.responseXML);
                            }
                            else {
                                document.querySelector("[data-m-segment='alert']").addClass("danger").innerHTML = `<p>${(request.responseText != null && request.responseText != "") ? request.responseText : "Error: A problem has occurred while attempting to complete the last operation!"}</p>`;
                                document.querySelector("[data-m-segment='alert']").show();
                            }
                        }
                        if (always !== undefined) {
                            always(request);
                        }
                    }
                };
                request.setRequestHeader("Content-Type", contentType);
                if (url.contains("localhost")) {
                    request.setRequestHeader("Cache-Control", "max-age=0");
                }
                if (headers != null) {
                    for (let k in headers) {
                        if (headers.hasOwnProperty(k)) {
                            request.setRequestHeader(k, headers[k]);
                        }
                    }
                }
                request.send(data);
            }
            let request = new XMLHttpRequest();
            let requestData = (typeof (data) !== "string") ? web.querystringify(data, true) : data;
            if (requestData.startsWith("?")) {
                requestData = requestData.substr(1);
            }
            if (requestData.endsWith("?")) {
                requestData = requestData.substr(0, requestData.length - 2);
            }
            var self = {
                url: url,
                method: method,
                contentType: contentType,
                dataType: dataType,
                data: requestData,
                async: async,
                request: request,
                send: function () {
                    _send(request, requestData);
                }
            };
            if (success != null || failure != null || always != null) {
                self.send();
            }
            return self;
        }
        web.ajax = ajax;
        function get(url, params = {}, contentType = "application/x-www-form-urlencoded; charset=UTF-8", dataType, success, failure, always, headers) {
            return ajax(url, params, "GET", true, (contentType != null) ? contentType : "application/x-www-form-urlencoded; charset=UTF-8", dataType, success, failure, always);
        }
        web.get = get;
        function post(url, params = {}, contentType = "application/x-www-form-urlencoded; charset=UTF-8", dataType, success, failure, always, headers) {
            return ajax(url, params, "POST", true, (contentType != null) ? contentType : "application/x-www-form-urlencoded; charset=UTF-8", dataType, success, failure, always);
        }
        web.post = post;
        function postAll(url, params = {}, contentType = "application/json;charset=utf-8", dataType, success, failure, always, headers) {
            return ajax(url, JSON.stringify({ "data": params }), "POST", true, (contentType != null) ? contentType : "application/json;charset=utf-8", dataType, success, failure, always);
        }
        web.postAll = postAll;
        function put(url, params = {}, contentType = "application/x-www-form-urlencoded; charset=UTF-8", dataType, success, failure, always, headers) {
            return ajax(url, params, "PUT", true, (contentType != null) ? contentType : "application/x-www-form-urlencoded; charset=UTF-8", dataType, success, failure, always);
        }
        web.put = put;
        function putAll(url, params = {}, contentType = "application/json;charset=utf-8", dataType, success, failure, always, headers) {
            return ajax(url, JSON.stringify({ "data": params }), "PUT", true, (contentType != null) ? contentType : "application/json;charset=utf-8", dataType, success, failure, always);
        }
        web.putAll = putAll;
        function remove(url, params = {}, contentType = "application/x-www-form-urlencoded; charset=UTF-8", dataType, success, failure, always, headers) {
            return ajax(url, params, "DELETE", true, (contentType != null) ? contentType : "application/x-www-form-urlencoded; charset=UTF-8", dataType, function (data) {
                if (data != null && data instanceof Array && data.length > 0) {
                    success(data[0]);
                }
                else {
                    success(data);
                }
            }, failure, always);
        }
        web.remove = remove;
    })(web = exports.web || (exports.web = {}));
});
//# sourceMappingURL=web.js.map